@@ -17,6 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-jpegdec
+ *
+ * Decodes jpeg images.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,13 +37,39 @@ static const GstElementDetails gst_jpeg_dec_details =
 GST_ELEMENT_DETAILS ("JPEG image decoder",
     "Codec/Decoder/Image",
     "Decode images from JPEG format",
-    "Wim Taymans <wim.taymans@tvd.be>");
+    "Wim Taymans <wim@fluendo.com>");
 
 #define MIN_WIDTH  16
 #define MAX_WIDTH  4096
-#define MIN_HEIGHT 16
+#define MIN_HEIGHT 8
 #define MAX_HEIGHT 4096
 
+#define DEFAULT_IDCT_METHOD	JDCT_FASTEST
+
+enum
+{
+  PROP_0,
+  PROP_IDCT_METHOD
+};
+
+#define GST_TYPE_IDCT_METHOD (gst_idct_method_get_type())
+static GType
+gst_idct_method_get_type (void)
+{
+  static GType idct_method_type = 0;
+  static const GEnumValue idct_method[] = {
+    {JDCT_ISLOW, "Slow but accurate integer algorithm", "islow"},
+    {JDCT_IFAST, "Faster, less accurate integer method", "ifast"},
+    {JDCT_FLOAT, "Floating-point: accurate, fast on fast HW", "float"},
+    {0, NULL, NULL},
+  };
+
+  if (!idct_method_type) {
+    idct_method_type = g_enum_register_static ("GstIDCTMethod", idct_method);
+  }
+  return idct_method_type;
+}
+
 static GstStaticPadTemplate gst_jpeg_dec_src_pad_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -77,8 +108,14 @@ static void gst_jpeg_dec_base_init (gpointer g_class);
 static void gst_jpeg_dec_class_init (GstJpegDecClass * klass);
 static void gst_jpeg_dec_init (GstJpegDec * jpegdec);
 
+static void gst_jpeg_dec_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_jpeg_dec_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
 static GstFlowReturn gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_jpeg_dec_setcaps (GstPad * pad, GstCaps * caps);
+static gboolean gst_jpeg_dec_sink_event (GstPad * pad, GstEvent * event);
 static GstStateChangeReturn gst_jpeg_dec_change_state (GstElement * element,
     GstStateChange transition);
 
@@ -116,6 +153,9 @@ gst_jpeg_dec_finalize (GObject * object)
   if (dec->tempbuf)
     gst_buffer_unref (dec->tempbuf);
 
+  if (dec->segment)
+    gst_segment_free (dec->segment);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -143,6 +183,13 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
   parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gst_jpeg_dec_finalize;
+  gobject_class->set_property = gst_jpeg_dec_set_property;
+  gobject_class->get_property = gst_jpeg_dec_get_property;
+
+  g_object_class_install_property (gobject_class, PROP_IDCT_METHOD,
+      g_param_spec_enum ("idct-method", "IDCT Method",
+          "The IDCT algorithm to use", GST_TYPE_IDCT_METHOD,
+          DEFAULT_IDCT_METHOD, G_PARAM_READWRITE));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_change_state);
@@ -150,7 +197,7 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
   GST_DEBUG_CATEGORY_INIT (jpeg_dec_debug, "jpegdec", 0, "JPEG decoder");
 }
 
-static gboolean
+static boolean
 gst_jpeg_dec_fill_input_buffer (j_decompress_ptr cinfo)
 {
 /*
@@ -188,7 +235,7 @@ gst_jpeg_dec_skip_input_data (j_decompress_ptr cinfo, glong num_bytes)
   }
 }
 
-static gboolean
+static boolean
 gst_jpeg_dec_resync_to_restart (j_decompress_ptr cinfo, gint desired)
 {
   GST_DEBUG ("resync_to_start");
@@ -238,11 +285,16 @@ gst_jpeg_dec_init (GstJpegDec * dec)
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_setcaps));
   gst_pad_set_chain_function (dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_jpeg_dec_chain));
+  gst_pad_set_event_function (dec->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_jpeg_dec_sink_event));
 
   dec->srcpad =
       gst_pad_new_from_static_template (&gst_jpeg_dec_src_pad_template, "src");
+  gst_pad_use_fixed_caps (dec->srcpad);
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
+  dec->segment = gst_segment_new ();
+
   /* setup jpeglib */
   memset (&dec->cinfo, 0, sizeof (dec->cinfo));
   memset (&dec->jerr, 0, sizeof (dec->jerr));
@@ -260,6 +312,9 @@ gst_jpeg_dec_init (GstJpegDec * dec)
   dec->cinfo.src->resync_to_restart = gst_jpeg_dec_resync_to_restart;
   dec->cinfo.src->term_source = gst_jpeg_dec_term_source;
   dec->jsrc.dec = dec;
+
+  /* init properties */
+  dec->idct_method = DEFAULT_IDCT_METHOD;
 }
 
 static inline gboolean
@@ -417,7 +472,7 @@ gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
         ++data;
         continue;
       }
-      GST_DEBUG ("found sync at 0x%08x", data - size);
+      GST_DEBUG ("found sync at %p", data - size);
     }
     while (*data == 0xff)
       ++data;
@@ -610,23 +665,42 @@ gst_jpeg_dec_setcaps (GstPad * pad, GstCaps * caps)
   return TRUE;
 }
 
+/* yuk */
+static void
+hresamplecpy1 (guint8 * dest, const guint8 * src, guint len)
+{
+  gint i;
+
+  for (i = 0; i < len; ++i) {
+    /* equivalent to: dest[i] = src[i << 1] */
+    *dest = *src;
+    ++dest;
+    ++src;
+    ++src;
+  }
+}
+
 static void
 gst_jpeg_dec_decode_indirect (GstJpegDec * dec, guchar * base[3],
-    guchar * last[3], guint width, guint height, gint r_v)
+    guchar * last[3], guint width, guint height, gint r_v, gint r_h)
 {
   guchar y[16][MAX_WIDTH];
-  guchar u[8][MAX_WIDTH / 2];
-  guchar v[8][MAX_WIDTH / 2];
+  guchar u[16][MAX_WIDTH];
+  guchar v[16][MAX_WIDTH];
   guchar *y_rows[16] = { y[0], y[1], y[2], y[3], y[4], y[5], y[6], y[7],
     y[8], y[9], y[10], y[11], y[12], y[13], y[14], y[15]
   };
-  guchar *u_rows[8] = { u[0], u[1], u[2], u[3], u[4], u[5], u[6], u[7] };
-  guchar *v_rows[8] = { v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7] };
+  guchar *u_rows[16] = { u[0], u[1], u[2], u[3], u[4], u[5], u[6], u[7],
+    u[8], u[9], u[10], u[11], u[12], u[13], u[14], u[15]
+  };
+  guchar *v_rows[16] = { v[0], v[1], v[2], v[3], v[4], v[5], v[6], v[7],
+    v[8], v[9], v[10], v[11], v[12], v[13], v[14], v[15]
+  };
   guchar **scanarray[3] = { y_rows, u_rows, v_rows };
   gint i, j, k;
 
   GST_DEBUG_OBJECT (dec,
-      "unadvantageous width, taking slow route involving memcpy");
+      "unadvantageous width or r_h, taking slow route involving memcpy");
 
   for (i = 0; i < height; i += r_v * DCTSIZE) {
     jpeg_read_raw_data (&dec->cinfo, scanarray, r_v * DCTSIZE);
@@ -639,8 +713,16 @@ gst_jpeg_dec_decode_indirect (GstJpegDec * dec, guchar * base[3],
         if (base[0] < last[0])
           base[0] += I420_Y_ROWSTRIDE (width);
       }
-      memcpy (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
-      memcpy (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
+      if (G_LIKELY (r_h == 2)) {
+        memcpy (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
+        memcpy (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
+      } else if (G_UNLIKELY (r_h == 1)) {
+        hresamplecpy1 (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
+        hresamplecpy1 (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
+      } else {
+        /* FIXME: implement (at least we avoid crashing by doing nothing) */
+      }
+
       if (r_v == 2 || (k & 1) != 0) {
         if (base[1] < last[1] && base[2] < last[2]) {
           base[1] += I420_U_ROWSTRIDE (width);
@@ -656,11 +738,14 @@ gst_jpeg_dec_decode_direct (GstJpegDec * dec, guchar * base[3],
     guchar * last[3], guint width, guint height, gint r_v)
 {
   guchar **line[3];             /* the jpeg line buffer */
+  guchar *y[4 * DCTSIZE];       /* alloc enough for the lines */
+  guchar *u[4 * DCTSIZE];
+  guchar *v[4 * DCTSIZE];
   gint i, j, k;
 
-  line[0] = g_new0 (guchar *, (r_v * DCTSIZE));
-  line[1] = g_new0 (guchar *, (r_v * DCTSIZE));
-  line[2] = g_new0 (guchar *, (r_v * DCTSIZE));
+  line[0] = y;
+  line[1] = u;
+  line[2] = v;
 
   /* let jpeglib decode directly into our final buffer */
   GST_DEBUG_OBJECT (dec, "decoding directly into output buffer");
@@ -685,13 +770,8 @@ gst_jpeg_dec_decode_direct (GstJpegDec * dec, guchar * base[3],
     }
     jpeg_read_raw_data (&dec->cinfo, line, r_v * DCTSIZE);
   }
-
-  g_free (line[0]);
-  g_free (line[1]);
-  g_free (line[2]);
 }
 
-
 static GstFlowReturn
 gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 {
@@ -701,27 +781,20 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   gulong size;
   guchar *data, *outdata;
   guchar *base[3], *last[3];
-  guint img_len;
+  guint img_len, outsize;
   gint width, height;
   gint r_h, r_v;
   gint i;
   guint code;
   GstClockTime timestamp, duration;
 
-  dec = GST_JPEG_DEC (GST_OBJECT_PARENT (pad));
+  dec = GST_JPEG_DEC (gst_pad_get_parent (pad));
 
   timestamp = GST_BUFFER_TIMESTAMP (buf);
   duration = GST_BUFFER_DURATION (buf);
 
-/*
-  GST_LOG_OBJECT (dec, "Received buffer: %d bytes, ts=%" GST_TIME_FORMAT
-      ", dur=%" GST_TIME_FORMAT, GST_BUFFER_SIZE (buf),
-      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
-*/
-
-  if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
+  if (GST_CLOCK_TIME_IS_VALID (timestamp))
     dec->next_ts = timestamp;
-  }
 
   if (dec->tempbuf) {
     dec->tempbuf = gst_buffer_join (dec->tempbuf, buf);
@@ -767,27 +840,34 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 
   GST_LOG_OBJECT (dec, "reading header %02x %02x %02x %02x", data[0], data[1],
       data[2], data[3]);
+
+  /* read header */
   jpeg_read_header (&dec->cinfo, TRUE);
 
   r_h = dec->cinfo.cur_comp_info[0]->h_samp_factor;
   r_v = dec->cinfo.cur_comp_info[0]->v_samp_factor;
 
-  GST_DEBUG ("num_components=%d, comps_in_scan=%d\n",
+  GST_DEBUG ("r_h = %d, r_v = %d", r_h, r_v);
+  GST_DEBUG ("num_components=%d, comps_in_scan=%d",
       dec->cinfo.num_components, dec->cinfo.comps_in_scan);
-  for (i = 0; i < dec->cinfo.num_components; ++i) {
-    GST_DEBUG ("[%d] h_samp_factor=%d, v_samp_factor=%d\n", i,
+
+  for (i = 0; i < dec->cinfo.comps_in_scan; ++i) {
+    GST_DEBUG ("[%d] h_samp_factor=%d, v_samp_factor=%d", i,
         dec->cinfo.cur_comp_info[i]->h_samp_factor,
         dec->cinfo.cur_comp_info[i]->v_samp_factor);
   }
 
+  /* prepare for raw output */
   dec->cinfo.do_fancy_upsampling = FALSE;
   dec->cinfo.do_block_smoothing = FALSE;
   dec->cinfo.out_color_space = JCS_YCbCr;
-  dec->cinfo.dct_method = JDCT_IFAST;
+  dec->cinfo.dct_method = dec->idct_method;
   dec->cinfo.raw_data_out = TRUE;
+
   GST_LOG_OBJECT (dec, "starting decompress");
   guarantee_huff_tables (&dec->cinfo);
   jpeg_start_decompress (&dec->cinfo);
+
   width = dec->cinfo.output_width;
   height = dec->cinfo.output_height;
 
@@ -817,7 +897,6 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     GST_DEBUG_OBJECT (dec, "max_v_samp_factor=%d",
         dec->cinfo.max_v_samp_factor);
 
-    gst_pad_use_fixed_caps (dec->srcpad);
     gst_pad_set_caps (dec->srcpad, caps);
     gst_caps_unref (caps);
 
@@ -825,14 +904,20 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     dec->caps_height = height;
     dec->caps_framerate_numerator = dec->framerate_numerator;
     dec->caps_framerate_denominator = dec->framerate_denominator;
+    dec->outsize = I420_SIZE (width, height);
   }
 
   ret = gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
-      I420_SIZE (width, height), GST_PAD_CAPS (dec->srcpad), &outbuf);
+      dec->outsize, GST_PAD_CAPS (dec->srcpad), &outbuf);
   if (ret != GST_FLOW_OK)
     goto alloc_failed;
 
   outdata = GST_BUFFER_DATA (outbuf);
+  outsize = GST_BUFFER_SIZE (outbuf);
+
+  GST_LOG_OBJECT (dec, "width %d, height %d, buffer size %d, required size %d",
+      width, height, outsize, dec->outsize);
+
   GST_BUFFER_TIMESTAMP (outbuf) = dec->next_ts;
 
   if (dec->packetized && GST_CLOCK_TIME_IS_VALID (dec->next_ts)) {
@@ -853,9 +938,6 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   }
   GST_BUFFER_DURATION (outbuf) = duration;
 
-  GST_LOG_OBJECT (dec, "width %d, height %d, buffer size %d, required size %d",
-      width, height, GST_BUFFER_SIZE (outbuf), I420_SIZE (width, height));
-
   /* mind the swap, jpeglib outputs blue chroma first */
   base[0] = outdata + I420_Y_OFFSET (width, height);
   base[1] = outdata + I420_U_OFFSET (width, height);
@@ -879,8 +961,8 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
    * copy over the data into our final picture buffer, otherwise jpeglib might
    * write over the end of a line into the beginning of the next line,
    * resulting in blocky artifacts on the left side of the picture. */
-  if (width % (dec->cinfo.max_h_samp_factor * DCTSIZE) != 0) {
-    gst_jpeg_dec_decode_indirect (dec, base, last, width, height, r_v);
+  if (r_h != 2 || width % (dec->cinfo.max_h_samp_factor * DCTSIZE) != 0) {
+    gst_jpeg_dec_decode_indirect (dec, base, last, width, height, r_v, r_h);
   } else {
     gst_jpeg_dec_decode_direct (dec, base, last, width, height, r_v);
   }
@@ -888,6 +970,32 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   GST_LOG_OBJECT (dec, "decompressing finished");
   jpeg_finish_decompress (&dec->cinfo);
 
+  /* Clipping */
+  if (dec->segment->format == GST_FORMAT_TIME) {
+    gint64 start, stop, clip_start, clip_stop;
+
+    GST_LOG_OBJECT (dec, "Attempting clipping");
+
+    start = GST_BUFFER_TIMESTAMP (outbuf);
+    if (GST_BUFFER_DURATION (outbuf) == GST_CLOCK_TIME_NONE)
+      stop = start;
+    else
+      stop = start + GST_BUFFER_DURATION (outbuf);
+
+    if (gst_segment_clip (dec->segment, GST_FORMAT_TIME,
+            start, stop, &clip_start, &clip_stop)) {
+      GST_LOG_OBJECT (dec, "Clipping start to %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (clip_start));
+      GST_BUFFER_TIMESTAMP (outbuf) = clip_start;
+      if (GST_BUFFER_DURATION (outbuf) != GST_CLOCK_TIME_NONE) {
+        GST_LOG_OBJECT (dec, "Clipping duration to %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (clip_stop - clip_start));
+        GST_BUFFER_DURATION (outbuf) = clip_stop - clip_start;
+      }
+    } else
+      goto drop_buffer;
+  }
+
   GST_LOG_OBJECT (dec, "pushing buffer (ts=%" GST_TIME_FORMAT ", dur=%"
       GST_TIME_FORMAT, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)));
@@ -905,13 +1013,18 @@ done:
     gst_buffer_unref (dec->tempbuf);
     dec->tempbuf = buf;
   }
+
+exit:
+  gst_object_unref (dec);
+
   return ret;
 
   /* special cases */
 need_more_data:
   {
     GST_LOG_OBJECT (dec, "we need more data");
-    return GST_FLOW_OK;
+    ret = GST_FLOW_OK;
+    goto exit;
   }
   /* ERRORS */
 wrong_size:
@@ -937,12 +1050,100 @@ alloc_failed:
     reason = gst_flow_get_name (ret);
 
     GST_DEBUG_OBJECT (dec, "failed to alloc buffer, reason %s", reason);
+    /* Reset for next time */
+    jpeg_abort_decompress (&dec->cinfo);
     if (GST_FLOW_IS_FATAL (ret)) {
       GST_ELEMENT_ERROR (dec, STREAM, DECODE,
           ("Buffer allocation failed, reason: %s", reason),
           ("Buffer allocation failed, reason: %s", reason));
     }
-    return ret;
+    goto exit;
+  }
+drop_buffer:
+  {
+    GST_WARNING_OBJECT (dec, "Outgoing buffer is outside configured segment");
+    gst_buffer_unref (outbuf);
+    ret = GST_FLOW_OK;
+    goto exit;
+  }
+}
+
+static gboolean
+gst_jpeg_dec_sink_event (GstPad * pad, GstEvent * event)
+{
+  gboolean ret = TRUE;
+  GstJpegDec *dec = GST_JPEG_DEC (GST_OBJECT_PARENT (pad));
+
+  GST_DEBUG_OBJECT (dec, "event : %s", GST_EVENT_TYPE_NAME (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_STOP:
+      GST_DEBUG_OBJECT (dec, "Aborting decompress");
+      jpeg_abort_decompress (&dec->cinfo);
+      break;
+    case GST_EVENT_NEWSEGMENT:{
+      gboolean update;
+      gdouble rate, applied_rate;
+      GstFormat format;
+      gint64 start, stop, position;
+
+      gst_event_parse_new_segment_full (event, &update, &rate, &applied_rate,
+          &format, &start, &stop, &position);
+
+      GST_DEBUG_OBJECT (dec, "Got NEWSEGMENT [%" GST_TIME_FORMAT
+          " - %" GST_TIME_FORMAT " / %" GST_TIME_FORMAT "]",
+          GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
+          GST_TIME_ARGS (position));
+
+      gst_segment_set_newsegment_full (dec->segment, update, rate,
+          applied_rate, format, start, stop, position);
+
+      break;
+    }
+    default:
+      break;
+  }
+
+  ret = gst_pad_push_event (dec->srcpad, event);
+
+  return ret;
+}
+
+static void
+gst_jpeg_dec_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstJpegDec *dec;
+
+  dec = GST_JPEG_DEC (object);
+
+  switch (prop_id) {
+    case PROP_IDCT_METHOD:
+      dec->idct_method = g_value_get_enum (value);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_jpeg_dec_get_property (GObject * object, guint prop_id, GValue * value,
+    GParamSpec * pspec)
+{
+  GstJpegDec *dec;
+
+  dec = GST_JPEG_DEC (object);
+
+  switch (prop_id) {
+    case PROP_IDCT_METHOD:
+      g_value_set_enum (value, dec->idct_method);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
@@ -963,6 +1164,7 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
       dec->caps_height = -1;
       dec->packetized = FALSE;
       dec->next_ts = 0;
+      gst_segment_init (dec->segment, GST_FORMAT_UNDEFINED);
     default:
       break;
   }
