@@ -33,7 +33,7 @@ GST_ELEMENT_DETAILS ("JPEG image encoder",
     "Encode images in JPEG format",
     "Wim Taymans <wim.taymans@tvd.be>");
 
-GST_DEBUG_CATEGORY (jpegenc_debug);
+GST_DEBUG_CATEGORY_STATIC (jpegenc_debug);
 #define GST_CAT_DEFAULT jpegenc_debug
 
 #define JPEG_DEFAULT_QUALITY 85
@@ -186,7 +186,7 @@ gst_jpegenc_init_destination (j_compress_ptr cinfo)
   GST_DEBUG ("gst_jpegenc_chain: init_destination");
 }
 
-static gboolean
+static boolean
 gst_jpegenc_flush_destination (j_compress_ptr cinfo)
 {
   GST_DEBUG ("gst_jpegenc_chain: flush_destination: buffer too small !!!");
@@ -264,12 +264,19 @@ gst_jpegenc_getcaps (GstPad * pad)
   /* we want to proxy properties like width, height and framerate from the
      other end of the element */
   otherpad = (pad == jpegenc->srcpad) ? jpegenc->sinkpad : jpegenc->srcpad;
-  caps = gst_pad_get_allowed_caps (otherpad);
+
+  caps = gst_pad_peer_get_caps (otherpad);
+  if (caps == NULL)
+    caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  else
+    caps = gst_caps_make_writable (caps);
+
   if (pad == jpegenc->srcpad) {
     name = "image/jpeg";
   } else {
     name = "video/x-raw-yuv";
   }
+
   for (i = 0; i < gst_caps_get_size (caps); i++) {
     structure = gst_caps_get_structure (caps, i);
 
@@ -413,7 +420,7 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
 
-  GST_DEBUG_OBJECT (jpegenc, "got buffer of %u bytes", size);
+  GST_DEBUG_OBJECT (jpegenc, "got buffer of %lu bytes", size);
 
   ret =
       gst_pad_alloc_buffer_and_set_caps (jpegenc->srcpad,
@@ -423,7 +430,7 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   if (ret != GST_FLOW_OK)
     goto done;
 
-  gst_buffer_stamp (outbuf, buf);
+  gst_buffer_copy_metadata (outbuf, buf, GST_BUFFER_COPY_TIMESTAMPS);
 
   width = jpegenc->width;
   height = jpegenc->height;
