@@ -16,6 +16,30 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-cacasink
+ * @see_also: #GstAASink
+ *
+ * <refsect2>
+ * <para>
+ * Displays video as color ascii art.
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * CACA_GEOMETRY=160x60 CACA_FONT=5x7 gst-launch filesrc location=test.avi ! decodebin ! ffmpegcolorspace ! cacasink
+ * </programlisting>
+ * This pipeline renders a video to ascii art into a separate window using a
+ * small font and specifying the ascii resolution.
+ * </para>
+ * <para>
+ * <programlisting>
+ * CACA_DRIVER=ncurses gst-launch filesrc location=test.avi ! decodebin ! ffmpegcolorspace ! cacasink
+ * </programlisting>
+ * This pipeline renders a video to ascii art into the current terminal.
+ * </para>
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -107,7 +131,7 @@ gst_cacasink_dither_get_type (void)
 {
   static GType dither_type = 0;
 
-  static GEnumValue dither_types[] = {
+  static const GEnumValue dither_types[] = {
     {CACA_DITHERING_NONE, "No dithering", "none"},
     {CACA_DITHERING_ORDERED2, "Ordered 2x2 Bayer dithering", "2x2"},
     {CACA_DITHERING_ORDERED4, "Ordered 4x4 Bayer dithering", "4x4"},
@@ -149,13 +173,19 @@ gst_cacasink_class_init (GstCACASinkClass * klass)
   gobject_class->get_property = gst_cacasink_get_property;
   gstelement_class->change_state = gst_cacasink_change_state;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SCREEN_WIDTH, g_param_spec_int ("screen-width", "screen_width", "screen_width", G_MININT, G_MAXINT, 0, G_PARAM_READABLE));       /* CHECKME */
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SCREEN_HEIGHT, g_param_spec_int ("screen-height", "screen_height", "screen_height", G_MININT, G_MAXINT, 0, G_PARAM_READABLE));   /* CHECKME */
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SCREEN_WIDTH,
+      g_param_spec_int ("screen-width", "Screen Width",
+          "The width of the screen", 0, G_MAXINT, GST_CACA_DEFAULT_SCREEN_WIDTH,
+          G_PARAM_READABLE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SCREEN_HEIGHT,
+      g_param_spec_int ("screen-height", "Screen Height",
+          "The height of the screen", 0, G_MAXINT,
+          GST_CACA_DEFAULT_SCREEN_HEIGHT, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DITHER,
       g_param_spec_enum ("dither", "Dither Type", "Set type of Dither",
           GST_TYPE_CACADITHER, CACA_DITHERING_NONE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ANTIALIASING,
-      g_param_spec_boolean ("anti-aliasing", "Anti-Aliasing",
+      g_param_spec_boolean ("anti-aliasing", "Anti Aliasing",
           "Enables Anti-Aliasing", TRUE, G_PARAM_READWRITE));
 
   gstbasesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_cacasink_setcaps);
@@ -275,9 +305,7 @@ gst_cacasink_set_property (GObject * object, guint prop_id,
       cacasink->antialiasing = g_value_get_boolean (value);
       if (cacasink->antialiasing) {
         caca_set_feature (CACA_ANTIALIASING_MAX);
-      }
-
-      else {
+      } else {
         caca_set_feature (CACA_ANTIALIASING_MIN);
       }
       break;
@@ -350,7 +378,6 @@ gst_cacasink_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
 
-
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       if (!gst_cacasink_open (GST_CACASINK (element)))
