@@ -52,6 +52,7 @@ enum
   ARG_IP,                       /* the ip of the server */
   ARG_PORT,                     /* the encoder port number on the server */
   ARG_PASSWORD,                 /* the encoder password on the server */
+  ARG_USERNAME,                 /* the encoder username on the server */
   ARG_PUBLIC,                   /* is this stream public? */
   ARG_STREAMNAME,               /* Name of the stream */
   ARG_DESCRIPTION,              /* Description of the stream */
@@ -66,6 +67,7 @@ enum
 #define DEFAULT_IP           "127.0.0.1"
 #define DEFAULT_PORT         8000
 #define DEFAULT_PASSWORD     "hackme"
+#define DEFAULT_USERNAME     "source"
 #define DEFAULT_STREAMNAME   ""
 #define DEFAULT_DESCRIPTION  ""
 #define DEFAULT_GENRE        ""
@@ -84,6 +86,7 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 static void gst_shout2send_class_init (GstShout2sendClass * klass);
 static void gst_shout2send_base_init (GstShout2sendClass * klass);
 static void gst_shout2send_init (GstShout2send * shout2send);
+static void gst_shout2send_finalize (GstShout2send * shout2send);
 
 static gboolean gst_shout2send_event (GstBaseSink * sink, GstEvent * event);
 static GstFlowReturn gst_shout2send_render (GstBaseSink * sink,
@@ -105,7 +108,7 @@ static GType
 gst_shout2send_protocol_get_type (void)
 {
   static GType shout2send_protocol_type = 0;
-  static GEnumValue shout2send_protocol[] = {
+  static const GEnumValue shout2send_protocol[] = {
     {SHOUT2SEND_PROTOCOL_XAUDIOCAST,
         "Xaudiocast Protocol (icecast 1.3.x)", "xaudiocast"},
     {SHOUT2SEND_PROTOCOL_ICY, "Icy Protocol (ShoutCast)", "icy"},
@@ -182,6 +185,7 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
 
   gobject_class->set_property = gst_shout2send_set_property;
   gobject_class->get_property = gst_shout2send_get_property;
+  gobject_class->finalize = (GObjectFinalizeFunc) gst_shout2send_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_IP,
       g_param_spec_string ("ip", "ip", "ip", DEFAULT_IP, G_PARAM_READWRITE));
@@ -193,6 +197,10 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
       g_param_spec_string ("password", "password", "password", DEFAULT_PASSWORD,
           G_PARAM_READWRITE));
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USERNAME,
+      g_param_spec_string ("username", "username", "username", DEFAULT_USERNAME,
+          G_PARAM_READWRITE));
+
   /* metadata */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_STREAMNAME,
       g_param_spec_string ("streamname", "streamname", "name of the stream",
@@ -244,6 +252,7 @@ gst_shout2send_init (GstShout2send * shout2send)
   shout2send->ip = g_strdup (DEFAULT_IP);
   shout2send->port = DEFAULT_PORT;
   shout2send->password = g_strdup (DEFAULT_PASSWORD);
+  shout2send->username = g_strdup (DEFAULT_USERNAME);
   shout2send->streamname = g_strdup (DEFAULT_STREAMNAME);
   shout2send->description = g_strdup (DEFAULT_DESCRIPTION);
   shout2send->genre = g_strdup (DEFAULT_GENRE);
@@ -256,14 +265,37 @@ gst_shout2send_init (GstShout2send * shout2send)
   shout2send->audio_format = SHOUT_FORMAT_VORBIS;
   shout2send->connected = FALSE;
   shout2send->songmetadata = NULL;
+  shout2send->songartist = NULL;
+  shout2send->songtitle = NULL;
+}
+
+static void
+gst_shout2send_finalize (GstShout2send * shout2send)
+{
+  g_free (shout2send->ip);
+  g_free (shout2send->password);
+  g_free (shout2send->username);
+  g_free (shout2send->streamname);
+  g_free (shout2send->description);
+  g_free (shout2send->genre);
+  g_free (shout2send->mount);
+  g_free (shout2send->url);
+
+  gst_tag_list_free (shout2send->tags);
+
+  G_OBJECT_CLASS (parent_class)->finalize ((GObject *) (shout2send));
 }
 
 static void
 set_shout_metadata (const GstTagList * list, const gchar * tag,
     gpointer user_data)
 {
-  char **shout_metadata = (char **) user_data;
-  gchar *value, *temp;
+  GstShout2send *shout2send = (GstShout2send *) user_data;
+  char **shout_metadata = &(shout2send->songmetadata);
+  char **song_artist = &(shout2send->songartist);
+  char **song_title = &(shout2send->songtitle);
+
+  gchar *value;
 
   GST_DEBUG ("tag: %s being added", tag);
   if (strcmp (tag, GST_TAG_ARTIST) == 0) {
@@ -272,15 +304,11 @@ set_shout_metadata (const GstTagList * list, const gchar * tag,
         GST_DEBUG ("Error reading \"%s\" tag value", tag);
         return;
       }
-      /* shout_metadata should be NULL if title is after artist in  list */
-      if (*shout_metadata == NULL) {
-        *shout_metadata = g_strdup (value);
-      } else {
-        temp = g_strdup_printf ("%s - %s", value, *shout_metadata);
-        g_free (*shout_metadata);
-        *shout_metadata = temp;
-      }
 
+      if (*song_artist != NULL)
+        g_free (*song_artist);
+
+      *song_artist = g_strdup (value);
     }
   } else if (strcmp (tag, GST_TAG_TITLE) == 0) {
     if (gst_tag_get_type (tag) == G_TYPE_STRING) {
@@ -288,16 +316,28 @@ set_shout_metadata (const GstTagList * list, const gchar * tag,
         GST_DEBUG ("Error reading \"%s\" tag value", tag);
         return;
       }
-      /* shout_metadata should be NULL if title is before artist in  list */
-      if (*shout_metadata == NULL) {
-        *shout_metadata = g_strdup (value);
-      } else {
-        temp = g_strdup_printf ("%s - %s", *shout_metadata, value);
-        g_free (*shout_metadata);
-        *shout_metadata = temp;
-      }
+
+      if (*song_title != NULL)
+        g_free (*song_title);
+
+      *song_title = g_strdup (value);
     }
   }
+
+  if (*shout_metadata != NULL)
+    g_free (*shout_metadata);
+
+
+  if (*song_title && *song_artist) {
+    *shout_metadata = g_strdup_printf ("%s - %s", *song_artist, *song_title);
+  } else if (*song_title && *song_artist == NULL) {
+    *shout_metadata = g_strdup_printf ("Unknown - %s", *song_title);
+  } else if (*song_title == NULL && *song_artist) {
+    *shout_metadata = g_strdup_printf ("%s - Unknown", *song_artist);
+  } else {
+    *shout_metadata = g_strdup_printf ("Unknown - Unknown");
+  }
+
   GST_LOG ("shout metadata is now: %s", *shout_metadata);
 }
 
@@ -355,8 +395,8 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
             list,
             gst_tag_setter_get_tag_merge_mode (GST_TAG_SETTER (shout2send)));
         /* lets get the artist and song tags */
-        gst_tag_list_foreach ((GstTagList *) shout2send->tags,
-            set_shout_metadata, &shout2send->songmetadata);
+        gst_tag_list_foreach ((GstTagList *) list,
+            set_shout_metadata, shout2send);
         if (shout2send->songmetadata && shout2send->connected) {
           shout_metadata_t *pmetadata;
 
@@ -450,9 +490,9 @@ gst_shout2send_start (GstBaseSink * basesink)
   if (shout_set_mount (sink->conn, sink->mount) != SHOUTERR_SUCCESS)
     goto set_failed;
 
-  cur_prop = "user";
+  cur_prop = "username";
   GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, "source");
-  if (shout_set_user (sink->conn, "source") != SHOUTERR_SUCCESS)
+  if (shout_set_user (sink->conn, sink->username) != SHOUTERR_SUCCESS)
     goto set_failed;
 
   version_string = gst_version_string ();
@@ -604,6 +644,11 @@ gst_shout2send_set_property (GObject * object, guint prop_id,
         g_free (shout2send->password);
       shout2send->password = g_strdup (g_value_get_string (value));
       break;
+    case ARG_USERNAME:
+      if (shout2send->username)
+        g_free (shout2send->username);
+      shout2send->username = g_strdup (g_value_get_string (value));
+      break;
     case ARG_STREAMNAME:       /* Name of the stream */
       if (shout2send->streamname)
         g_free (shout2send->streamname);
@@ -656,6 +701,9 @@ gst_shout2send_get_property (GObject * object, guint prop_id,
     case ARG_PASSWORD:
       g_value_set_string (value, shout2send->password);
       break;
+    case ARG_USERNAME:
+      g_value_set_string (value, shout2send->username);
+      break;
     case ARG_STREAMNAME:       /* Name of the stream */
       g_value_set_string (value, shout2send->streamname);
       break;
