@@ -54,7 +54,7 @@
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("AudioInvert",
+GST_ELEMENT_DETAILS ("Audio inversion",
     "Filter/Effect/Audio",
     "Swaps upper and lower half of audio samples",
     "Sebastian Dröge <slomo@circular-chaos.org>");
@@ -147,8 +147,8 @@ static void
 gst_audio_invert_init (GstAudioInvert * filter, GstAudioInvertClass * klass)
 {
   filter->degree = 0.0;
-  filter->width = 0;
   gst_base_transform_set_in_place (GST_BASE_TRANSFORM (filter), TRUE);
+  gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (filter), TRUE);
 }
 
 static void
@@ -202,8 +202,6 @@ gst_audio_invert_setup (GstAudioFilter * base, GstRingBufferSpec * format)
   else
     ret = FALSE;
 
-  filter->width = format->width / 8;
-
   return ret;
 }
 
@@ -240,14 +238,16 @@ static GstFlowReturn
 gst_audio_invert_transform_ip (GstBaseTransform * base, GstBuffer * buf)
 {
   GstAudioInvert *filter = GST_AUDIO_INVERT (base);
-  guint num_samples = GST_BUFFER_SIZE (buf) / filter->width;
-
-  if (!gst_buffer_is_writable (buf))
-    return GST_FLOW_OK;
+  guint num_samples =
+      GST_BUFFER_SIZE (buf) / (GST_AUDIO_FILTER (filter)->format.width / 8);
 
   if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buf)))
     gst_object_sync_values (G_OBJECT (filter), GST_BUFFER_TIMESTAMP (buf));
 
+  if (gst_base_transform_is_passthrough (base) ||
+      G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_GAP)))
+    return GST_FLOW_OK;
+
   filter->process (filter, GST_BUFFER_DATA (buf), num_samples);
 
   return GST_FLOW_OK;
