@@ -210,7 +210,7 @@ gst_vertigotv_transform (GstBaseTransform * trans, GstBuffer * in,
 
   filter = GST_VERTIGOTV (trans);
 
-  gst_buffer_stamp (out, in);
+  gst_buffer_copy_metadata (out, in, GST_BUFFER_COPY_TIMESTAMPS);
 
   src = (guint32 *) GST_BUFFER_DATA (in);
   dest = (guint32 *) GST_BUFFER_DATA (out);
