@@ -47,12 +47,12 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch filesrc location=file.mp3 ! decodebin ! audioconvert ! "audio/x-raw-int,channels=2" ! deinterleave name=d  interleave name=i ! audioconvert ! wavenc ! filesink location=test.wav    d.src0 ! queue ! audioconvert ! i.sink1    d.src1 ! queue ! audioconvert ! i.sink0
+ * gst-launch-0.10 filesrc location=file.mp3 ! decodebin ! audioconvert ! "audio/x-raw-int,channels=2" ! deinterleave name=d  interleave name=i ! audioconvert ! wavenc ! filesink location=test.wav    d.src0 ! queue ! audioconvert ! i.sink1    d.src1 ! queue ! audioconvert ! i.sink0
  * ]| Decodes and deinterleaves a Stereo MP3 file into separate channels and
  * then interleaves the channels again to a WAV file with the channel with the
  * channels exchanged.
  * |[
- * gst-launch interleave name=i ! audioconvert ! wavenc ! filesink location=file.wav  filesrc location=file1.wav ! decodebin ! audioconvert ! "audio/x-raw-int,channels=1" ! queue ! i.sink0   filesrc location=file2.wav ! decodebin ! audioconvert ! "audio/x-raw-int,channels=1" ! queue ! i.sink1
+ * gst-launch-0.10 interleave name=i ! audioconvert ! wavenc ! filesink location=file.wav  filesrc location=file1.wav ! decodebin ! audioconvert ! "audio/x-raw-int,channels=1" ! queue ! i.sink0   filesrc location=file2.wav ! decodebin ! audioconvert ! "audio/x-raw-int,channels=1" ! queue ! i.sink1
  * ]| Interleaves two Mono WAV files to a single Stereo WAV file.
  * </refsect2>
  */
@@ -260,8 +260,11 @@ static gboolean
 gst_interleave_check_channel_positions (GValueArray * positions)
 {
   gint i;
+
   guint channels;
+
   GstAudioChannelPosition *pos;
+
   gboolean ret;
 
   channels = positions->n_values;
@@ -330,6 +333,7 @@ static void
 gst_interleave_class_init (GstInterleaveClass * klass)
 {
   GstElementClass *gstelement_class;
+
   GObjectClass *gobject_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
@@ -471,8 +475,11 @@ gst_interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
     const gchar * req_name)
 {
   GstInterleave *self = GST_INTERLEAVE (element);
+
   GstPad *new_pad;
+
   gchar *pad_name;
+
   gint channels, padnumber;
   GValue val = { 0, };
 
@@ -516,6 +523,7 @@ gst_interleave_request_new_pad (GstElement * element, GstPadTemplate * templ,
   /* Update the src caps if we already have them */
   if (self->sinkcaps) {
     GstCaps *srccaps;
+
     GstStructure *s;
 
     /* Take lock to make sure processing finishes first */
@@ -554,6 +562,7 @@ static void
 gst_interleave_release_pad (GstElement * element, GstPad * pad)
 {
   GstInterleave *self = GST_INTERLEAVE (element);
+
   GList *l;
 
   g_return_if_fail (GST_IS_INTERLEAVE_PAD (pad));
@@ -580,6 +589,7 @@ gst_interleave_release_pad (GstElement * element, GstPad * pad)
   if (self->sinkcaps) {
     if (self->channels > 0) {
       GstCaps *srccaps;
+
       GstStructure *s;
 
       srccaps = gst_caps_copy (self->sinkcaps);
@@ -606,6 +616,7 @@ static GstStateChangeReturn
 gst_interleave_change_state (GstElement * element, GstStateChange transition)
 {
   GstInterleave *self;
+
   GstStateChangeReturn ret;
 
   self = GST_INTERLEAVE (element);
@@ -657,6 +668,7 @@ static void
 __remove_channels (GstCaps * caps)
 {
   GstStructure *s;
+
   gint i, size;
 
   size = gst_caps_get_size (caps);
@@ -671,6 +683,7 @@ static void
 __set_channels (GstCaps * caps, gint channels)
 {
   GstStructure *s;
+
   gint i, size;
 
   size = gst_caps_get_size (caps);
@@ -688,6 +701,7 @@ static GstCaps *
 gst_interleave_sink_getcaps (GstPad * pad)
 {
   GstInterleave *self = GST_INTERLEAVE (gst_pad_get_parent (pad));
+
   GstCaps *result, *peercaps, *sinkcaps;
 
   GST_OBJECT_LOCK (self);
@@ -766,7 +780,9 @@ gst_interleave_sink_setcaps (GstPad * pad, GstCaps * caps)
     goto cannot_change_caps;
   } else {
     GstCaps *srccaps;
+
     GstStructure *s;
+
     gboolean res;
 
     s = gst_caps_get_structure (caps, 0);
@@ -809,6 +825,7 @@ gst_interleave_sink_setcaps (GstPad * pad, GstCaps * caps)
 
   if (!self->sinkcaps) {
     GstCaps *sinkcaps = gst_caps_copy (caps);
+
     GstStructure *s = gst_caps_get_structure (sinkcaps, 0);
 
     gst_structure_remove_field (s, "channel-positions");
@@ -824,14 +841,14 @@ gst_interleave_sink_setcaps (GstPad * pad, GstCaps * caps)
 
 cannot_change_caps:
   {
-    GST_WARNING_OBJECT (self, "caps of %" GST_PTR_FORMAT " already set, can't "
+    GST_DEBUG_OBJECT (self, "caps of %" GST_PTR_FORMAT " already set, can't "
         "change", self->sinkcaps);
     gst_object_unref (self);
     return FALSE;
   }
 src_did_not_accept:
   {
-    GST_WARNING_OBJECT (self, "src did not accept setcaps()");
+    GST_DEBUG_OBJECT (self, "src did not accept setcaps()");
     gst_object_unref (self);
     return FALSE;
   }
@@ -854,6 +871,7 @@ static gboolean
 gst_interleave_sink_event (GstPad * pad, GstEvent * event)
 {
   GstInterleave *self = GST_INTERLEAVE (gst_pad_get_parent (pad));
+
   gboolean ret;
 
   GST_DEBUG ("Got %s event on pad %s:%s", GST_EVENT_TYPE_NAME (event),
@@ -884,9 +902,13 @@ static gboolean
 gst_interleave_src_query_duration (GstInterleave * self, GstQuery * query)
 {
   gint64 max;
+
   gboolean res;
+
   GstFormat format;
+
   GstIterator *it;
+
   gboolean done;
 
   /* parse format */
@@ -962,9 +984,13 @@ static gboolean
 gst_interleave_src_query_latency (GstInterleave * self, GstQuery * query)
 {
   GstClockTime min, max;
+
   gboolean live;
+
   gboolean res;
+
   GstIterator *it;
+
   gboolean done;
 
   res = TRUE;
@@ -978,6 +1004,7 @@ gst_interleave_src_query_latency (GstInterleave * self, GstQuery * query)
   it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (self));
   while (!done) {
     GstIteratorResult ires;
+
     gpointer item;
 
     ires = gst_iterator_next (it, &item);
@@ -988,8 +1015,11 @@ gst_interleave_src_query_latency (GstInterleave * self, GstQuery * query)
       case GST_ITERATOR_OK:
       {
         GstPad *pad = GST_PAD_CAST (item);
+
         GstQuery *peerquery;
+
         GstClockTime min_cur, max_cur;
+
         gboolean live_cur;
 
         peerquery = gst_query_new_latency ();
@@ -1046,6 +1076,7 @@ static gboolean
 gst_interleave_src_query (GstPad * pad, GstQuery * query)
 {
   GstInterleave *self = GST_INTERLEAVE (gst_pad_get_parent (pad));
+
   gboolean res = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -1112,12 +1143,16 @@ forward_event_func (GstPad * pad, GValue * ret, GstEvent * event)
 static gboolean
 forward_event (GstInterleave * self, GstEvent * event)
 {
+  gboolean ret;
+
   GstIterator *it;
   GValue vret = { 0 };
 
   GST_LOG_OBJECT (self, "Forwarding event %p (%s)", event,
       GST_EVENT_TYPE_NAME (event));
 
+  ret = TRUE;
+
   g_value_init (&vret, G_TYPE_BOOLEAN);
   g_value_set_boolean (&vret, TRUE);
   it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (self));
@@ -1126,7 +1161,9 @@ forward_event (GstInterleave * self, GstEvent * event)
   gst_iterator_free (it);
   gst_event_unref (event);
 
-  return g_value_get_boolean (&vret);
+  ret = g_value_get_boolean (&vret);
+
+  return ret;
 }
 
 
@@ -1134,6 +1171,7 @@ static gboolean
 gst_interleave_src_event (GstPad * pad, GstEvent * event)
 {
   GstInterleave *self = GST_INTERLEAVE (gst_pad_get_parent (pad));
+
   gboolean result;
 
   switch (GST_EVENT_TYPE (event)) {
@@ -1144,7 +1182,9 @@ gst_interleave_src_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_SEEK:
     {
       GstSeekFlags flags;
+
       GstSeekType curtype;
+
       gint64 cur;
 
       /* parse the seek parameters */
@@ -1192,12 +1232,19 @@ static GstFlowReturn
 gst_interleave_collected (GstCollectPads * pads, GstInterleave * self)
 {
   guint size;
+
   GstBuffer *outbuf;
+
   GstFlowReturn ret = GST_FLOW_OK;
+
   GSList *collected;
+
   guint nsamples;
+
   guint ncollected = 0;
+
   gboolean empty = TRUE;
+
   gint width = self->width / 8;
 
   g_return_val_if_fail (self->func != NULL, GST_FLOW_NOT_NEGOTIATED);
@@ -1233,7 +1280,9 @@ gst_interleave_collected (GstCollectPads * pads, GstInterleave * self)
 
   for (collected = pads->data; collected != NULL; collected = collected->next) {
     GstCollectData *cdata;
+
     GstBuffer *inbuf;
+
     guint8 *outdata;
 
     cdata = (GstCollectData *) collected->data;
