@@ -25,6 +25,7 @@
 #endif
 
 #include <string.h>
+#include <gst/floatcast/floatcast.h>
 
 #include "ebml-write.h"
 #include "ebml-ids.h"
@@ -71,6 +72,11 @@ gst_ebml_write_finalize (GObject * object)
 
   gst_object_unref (ebml->srcpad);
 
+  if (ebml->cache) {
+    gst_buffer_unref (ebml->cache);
+    ebml->cache = NULL;
+  }
+
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -153,7 +159,7 @@ gst_ebml_last_write_result (GstEbmlWrite * ebml)
 void
 gst_ebml_write_set_cache (GstEbmlWrite * ebml, guint size)
 {
-  /* This is currently broken. I don't know why yet. */
+  /* FIXME: This is currently broken. I don't know why yet. */
   return;
 
   g_return_if_fail (ebml->cache == NULL);
@@ -250,6 +256,7 @@ static void
 gst_ebml_write_element_id (GstBuffer * buf, guint32 id)
 {
   guint8 *data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
+
   guint bytes = 4, mask = 0x10;
 
   /* get ID length */
@@ -285,6 +292,7 @@ static void
 gst_ebml_write_element_size (GstBuffer * buf, guint64 size)
 {
   guint8 *data = GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf);
+
   guint bytes = 1, mask = 0x80;
 
   if (size != GST_EBML_SIZE_UNKNOWN) {
@@ -371,6 +379,9 @@ gst_ebml_write_element_push (GstEbmlWrite * ebml, GstBuffer * buf)
     buf = gst_buffer_make_metadata_writable (buf);
     gst_buffer_set_caps (buf, GST_PAD_CAPS (ebml->srcpad));
     ebml->last_write_result = gst_pad_push (ebml->srcpad, buf);
+  } else {
+    if (buf != ebml->cache)
+      gst_buffer_unref (buf);
   }
 }
 
@@ -471,6 +482,7 @@ void
 gst_ebml_write_uint (GstEbmlWrite * ebml, guint32 id, guint64 num)
 {
   GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
+
   guint size = gst_ebml_write_get_uint_size (num);
 
   /* write */
@@ -499,6 +511,7 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
    * have a number (-)0x8000 (G_MINSHORT), then my abs()<<1
    * will be 0x10000; this is G_MAXUSHORT+1! So: if (<0) -1. */
   guint64 unum = (num < 0 ? (-num - 1) << 1 : num << 1);
+
   guint size = gst_ebml_write_get_uint_size (unum);
 
   /* make unsigned */
@@ -529,21 +542,12 @@ gst_ebml_write_sint (GstEbmlWrite * ebml, guint32 id, gint64 num)
 void
 gst_ebml_write_float (GstEbmlWrite * ebml, guint32 id, gdouble num)
 {
-#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
-  gint n;
-#endif
   GstBuffer *buf = gst_ebml_write_element_new (ebml, sizeof (num));
 
   gst_ebml_write_element_id (buf, id);
   gst_ebml_write_element_size (buf, 8);
-#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
-  for (n = 0; n < 8; n++) {
-    GST_BUFFER_DATA (buf)[GST_BUFFER_SIZE (buf)] = ((guint8 *) & num)[7 - n];
-    GST_BUFFER_SIZE (buf) += 1;
-  }
-#else
+  num = GDOUBLE_TO_BE (num);
   gst_ebml_write_element_data (buf, (guint8 *) & num, 8);
-#endif
   gst_ebml_write_element_push (ebml, buf);
 }
 
@@ -560,6 +564,7 @@ void
 gst_ebml_write_ascii (GstEbmlWrite * ebml, guint32 id, const gchar * str)
 {
   gint len = strlen (str) + 1;  /* add trailing '\0' */
+
   GstBuffer *buf = gst_ebml_write_element_new (ebml, len);
 
   gst_ebml_write_element_id (buf, id);
@@ -615,6 +620,7 @@ guint64
 gst_ebml_write_master_start (GstEbmlWrite * ebml, guint32 id)
 {
   guint64 pos = ebml->pos, t;
+
   GstBuffer *buf = gst_ebml_write_element_new (ebml, 0);
 
   t = GST_BUFFER_SIZE (buf);
@@ -638,6 +644,7 @@ void
 gst_ebml_write_master_finish (GstEbmlWrite * ebml, guint64 startpos)
 {
   guint64 pos = ebml->pos;
+
   GstBuffer *buf;
 
   gst_ebml_write_seek (ebml, startpos);
@@ -731,6 +738,7 @@ void
 gst_ebml_replace_uint (GstEbmlWrite * ebml, guint64 pos, guint64 num)
 {
   guint64 oldpos = ebml->pos;
+
   GstBuffer *buf = gst_buffer_new_and_alloc (8);
 
   gst_ebml_write_seek (ebml, pos);
