@@ -20,6 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* TODO: - check everywhere that we don't write invalid values
+ *       - make sure timestamps are correctly scaled everywhere
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -51,6 +55,14 @@ static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
   "height = (int) [ 16, 4096 ], " \
   "framerate = (fraction) [ 0, MAX ]"
 
+#define COMMON_VIDEO_CAPS_NO_FRAMERATE \
+  "width = (int) [ 16, 4096 ], " \
+  "height = (int) [ 16, 4096 ] "
+
+/* FIXME: 
+ * * require codec data, etc as needed
+ */
+
 static GstStaticPadTemplate videosink_templ =
     GST_STATIC_PAD_TEMPLATE ("video_%d",
     GST_PAD_SINK,
@@ -74,7 +86,7 @@ static GstStaticPadTemplate videosink_templ =
         "video/x-msmpeg, "
         COMMON_VIDEO_CAPS "; "
         "image/jpeg, "
-        COMMON_VIDEO_CAPS "; "
+        COMMON_VIDEO_CAPS_NO_FRAMERATE "; "
         "video/x-theora; "
         "video/x-dirac, "
         COMMON_VIDEO_CAPS "; "
@@ -88,6 +100,7 @@ static GstStaticPadTemplate videosink_templ =
 
 /* FIXME:
  * * audio/x-raw-float: endianness needs defining.
+ * * require codec data, etc as needed
  */
 static GstStaticPadTemplate audiosink_templ =
     GST_STATIC_PAD_TEMPLATE ("audio_%d",
@@ -180,11 +193,6 @@ static void
 gst_matroska_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  static const GstElementDetails gst_matroska_mux_details =
-      GST_ELEMENT_DETAILS ("Matroska muxer",
-      "Codec/Muxer",
-      "Muxes video/audio/subtitle streams into a matroska stream",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&videosink_templ));
@@ -194,7 +202,10 @@ gst_matroska_mux_base_init (gpointer g_class)
       gst_static_pad_template_get (&subtitlesink_templ));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
-  gst_element_class_set_details (element_class, &gst_matroska_mux_details);
+  gst_element_class_set_details_simple (element_class, "Matroska muxer",
+      "Codec/Muxer",
+      "Muxes video/audio/subtitle streams into a matroska stream",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
   GST_DEBUG_CATEGORY_INIT (matroskamux_debug, "matroskamux", 0,
       "Matroska muxer");
@@ -204,12 +215,13 @@ static void
 gst_matroska_mux_class_init (GstMatroskaMuxClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_matroska_mux_finalize);
+  gobject_class->finalize = gst_matroska_mux_finalize;
 
   gobject_class->get_property = gst_matroska_mux_get_property;
   gobject_class->set_property = gst_matroska_mux_set_property;
@@ -242,11 +254,7 @@ gst_matroska_mux_class_init (GstMatroskaMuxClass * klass)
 static void
 gst_matroska_mux_init (GstMatroskaMux * mux, GstMatroskaMuxClass * g_class)
 {
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
-
-  mux->srcpad =
-      gst_pad_new_from_template (gst_element_class_get_pad_template
-      (gstelement_class, "src"), "src");
+  mux->srcpad = gst_pad_new_from_static_template (&src_templ, "src");
   gst_pad_set_event_function (mux->srcpad, gst_matroska_mux_handle_src_event);
   gst_element_add_pad (GST_ELEMENT (mux), mux->srcpad);
 
@@ -297,8 +305,10 @@ static guint32
 gst_matroska_mux_create_uid (void)
 {
   guint32 uid = 0;
+
   GRand *rand = g_rand_new ();
 
+  /* FIXME: array needs locking or moved into instance structure */
   while (!uid) {
     guint i;
 
@@ -353,6 +363,7 @@ static void
 gst_matroska_mux_reset (GstElement * element)
 {
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
+
   GSList *walk;
 
   /* reset EBML write */
@@ -364,6 +375,7 @@ gst_matroska_mux_reset (GstElement * element)
   /* clean up existing streams */
   while ((walk = mux->collect->data) != NULL) {
     GstMatroskaPad *collect_pad;
+
     GstPad *thepad;
 
     collect_pad = (GstMatroskaPad *) walk->data;
@@ -392,7 +404,7 @@ gst_matroska_mux_reset (GstElement * element)
   mux->index = NULL;
 
   /* reset timers */
-  mux->time_scale = 1000000;
+  mux->time_scale = GST_MSECOND;
   mux->duration = 0;
 
   /* reset uid array */
@@ -407,11 +419,6 @@ gst_matroska_mux_reset (GstElement * element)
   mux->cluster_time = 0;
   mux->cluster_pos = 0;
 
-  /* reset meta-seek index */
-  mux->num_meta_indexes = 0;
-  g_free (mux->meta_index);
-  mux->meta_index = NULL;
-
   /* reset tags */
   if (mux->tags) {
     gst_tag_list_free (mux->tags);
@@ -459,9 +466,13 @@ static gboolean
 gst_matroska_mux_handle_sink_event (GstPad * pad, GstEvent * event)
 {
   GstMatroskaTrackContext *context;
+
   GstMatroskaPad *collect_pad;
+
   GstMatroskaMux *mux;
+
   GstTagList *list;
+
   gboolean ret;
 
   mux = GST_MATROSKA_MUX (gst_pad_get_parent (pad));
@@ -510,14 +521,21 @@ static gboolean
 gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstMatroskaTrackContext *context = NULL;
+
   GstMatroskaTrackVideoContext *videocontext;
+
   GstMatroskaMux *mux;
+
   GstMatroskaPad *collect_pad;
-  const gchar *mimetype;
-  gint width, height, pixel_width, pixel_height;
-  const GValue *framerate;
+
   GstStructure *structure;
 
+  const gchar *mimetype;
+
+  gint width, height, pixel_width, pixel_height;
+
+  gint fps_d, fps_n;
+
   mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
 
   /* find context */
@@ -541,16 +559,16 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
   /* get general properties */
   gst_structure_get_int (structure, "width", &width);
   gst_structure_get_int (structure, "height", &height);
-  framerate = gst_structure_get_value (structure, "framerate");
-  if (framerate == NULL || !GST_VALUE_HOLDS_FRACTION (framerate))
-    return FALSE;
-
   videocontext->pixel_width = width;
   videocontext->pixel_height = height;
-  context->default_duration = gst_util_uint64_scale_int (GST_SECOND,
-      gst_value_get_fraction_denominator (framerate),
-      gst_value_get_fraction_numerator (framerate));
-
+  if (gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d)) {
+    context->default_duration =
+        gst_util_uint64_scale_int (GST_SECOND, fps_d, fps_n);
+    GST_LOG_OBJECT (pad, "default duration = %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (context->default_duration));
+  } else {
+    context->default_duration = 0;
+  }
   if (gst_structure_get_fraction (structure, "pixel-aspect-ratio",
           &pixel_width, &pixel_height)) {
     if (pixel_width > pixel_height) {
@@ -571,9 +589,13 @@ gst_matroska_mux_video_pad_setcaps (GstPad * pad, GstCaps * caps)
 skip_details:
 
   videocontext->asr_mode = GST_MATROSKA_ASPECT_RATIO_MODE_FREE;
-  videocontext->eye_mode = GST_MATROSKA_EYE_MODE_MONO;
   videocontext->fourcc = 0;
 
+  /* TODO: - check if we handle all codecs by the spec, i.e. codec private
+   *         data and other settings
+   *       - add new formats
+   */
+
   /* find type */
   if (!strcmp (mimetype, "video/x-raw-yuv")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_UNCOMPRESSED);
@@ -588,10 +610,11 @@ skip_details:
       ||!strcmp (mimetype, "video/x-huffyuv")
       || !strcmp (mimetype, "video/x-divx")
       || !strcmp (mimetype, "video/x-dv")
-      || !strcmp (mimetype, "video/x-h263")
-      || !strcmp (mimetype, "video/x-dirac")) {
+      || !strcmp (mimetype, "video/x-h263")) {
     BITMAPINFOHEADER *bih;
+
     const GValue *codec_data;
+
     gint size = sizeof (BITMAPINFOHEADER);
 
     bih = g_new0 (BITMAPINFOHEADER, 1);
@@ -626,8 +649,6 @@ skip_details:
           GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DX50"));
           break;
       }
-    } else if (!strcmp (mimetype, "video/x-dirac")) {
-      GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("drac"));
     }
 
     /* process codec private/initialization data, if any */
@@ -665,6 +686,7 @@ skip_details:
 
     if (codec_data != NULL) {
       guint8 *priv_data = NULL;
+
       guint priv_data_size = 0;
 
       GstBuffer *codec_data_buf = g_value_peek_pointer (codec_data);
@@ -696,6 +718,10 @@ skip_details:
           ("theora stream headers missing or malformed"));
       return FALSE;
     }
+    return TRUE;
+  } else if (!strcmp (mimetype, "video/x-dirac")) {
+    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_DIRAC);
+
     return TRUE;
   } else if (!strcmp (mimetype, "video/mpeg")) {
     gint mpegversion;
@@ -730,8 +756,11 @@ xiph3_streamheader_to_codecdata (const GValue * streamheader,
     GstMatroskaTrackContext * context, GstBuffer ** p_buf0)
 {
   GstBuffer *buf[3];
+
   GArray *bufarr;
+
   guint8 *priv_data;
+
   guint i, offset, priv_data_size;
 
   if (streamheader == NULL)
@@ -830,6 +859,7 @@ vorbis_streamheader_to_codecdata (const GValue * streamheader,
   } else {
     if (memcmp (GST_BUFFER_DATA (buf0) + 1, "vorbis", 6) == 0) {
       GstMatroskaTrackAudioContext *audiocontext;
+
       guint8 *hdr;
 
       hdr = GST_BUFFER_DATA (buf0) + 1 + 6 + 4;
@@ -860,7 +890,9 @@ theora_streamheader_to_codecdata (const GValue * streamheader,
     GST_WARNING ("First header not a theora identification header, ignoring");
   } else {
     GstMatroskaTrackVideoContext *videocontext;
+
     guint fps_num, fps_denom, par_num, par_denom;
+
     guint8 *hdr;
 
     hdr = GST_BUFFER_DATA (buf0) + 1 + 6 + 3 + 2 + 2;
@@ -915,11 +947,17 @@ static gboolean
 gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstMatroskaTrackContext *context = NULL;
+
   GstMatroskaTrackAudioContext *audiocontext;
+
   GstMatroskaMux *mux;
+
   GstMatroskaPad *collect_pad;
+
   const gchar *mimetype;
+
   gint samplerate = 0, channels = 0;
+
   GstStructure *structure;
 
   mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
@@ -944,6 +982,11 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
   audiocontext->bitdepth = 0;
   context->default_duration = 0;
 
+  /* TODO: - check if we handle all codecs by the spec, i.e. codec private
+   *         data and other settings
+   *       - add new formats
+   */
+
   if (!strcmp (mimetype, "audio/mpeg")) {
     gint mpegversion = 0;
 
@@ -952,6 +995,10 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
       case 1:{
         gint layer;
 
+        /* FIXME: number of samples per frame also depends on the mpegversion
+         * which we don't pass as a caps field
+         */
+
         gst_structure_get_int (structure, "layer", &layer);
         switch (layer) {
           case 1:
@@ -975,10 +1022,12 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
         break;
       }
       case 2:
-        context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG2 "MAIN");
+        context->codec_id =
+            g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG2 "MAIN");
         break;
       case 4:
-        context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_MPEG4 "MAIN");
+        context->codec_id =
+            g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AAC_MPEG4 "MAIN");
         break;
       default:
         return FALSE;
@@ -987,6 +1036,7 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
     return TRUE;
   } else if (!strcmp (mimetype, "audio/x-raw-int")) {
     gint endianness, width, depth;
+
     gboolean signedness;
 
     if (!gst_structure_get_int (structure, "width", &width) ||
@@ -1073,7 +1123,8 @@ gst_matroska_mux_audio_pad_setcaps (GstPad * pad, GstCaps * caps)
 static gboolean
 gst_matroska_mux_subtitle_pad_setcaps (GstPad * pad, GstCaps * caps)
 {
-  /* Consider this as boilerplate code for now. There is
+  /* FIXME:
+   * Consider this as boilerplate code for now. There is
    * no single subtitle creation element in GStreamer,
    * neither do I know how subtitling works at all. */
 
@@ -1096,30 +1147,36 @@ gst_matroska_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * pad_name)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
+
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
+
   GstMatroskaPad *collect_pad;
+
   GstPad *newpad = NULL;
+
   gchar *name = NULL;
+
   GstPadSetCapsFunction setcapsfunc = NULL;
+
   GstMatroskaTrackContext *context = NULL;
 
   if (templ == gst_element_class_get_pad_template (klass, "audio_%d")) {
     name = g_strdup_printf ("audio_%d", mux->num_a_streams++);
-    setcapsfunc = gst_matroska_mux_audio_pad_setcaps;
+    setcapsfunc = GST_DEBUG_FUNCPTR (gst_matroska_mux_audio_pad_setcaps);
     context = (GstMatroskaTrackContext *)
         g_new0 (GstMatroskaTrackAudioContext, 1);
     context->type = GST_MATROSKA_TRACK_TYPE_AUDIO;
     context->name = g_strdup ("Audio");
   } else if (templ == gst_element_class_get_pad_template (klass, "video_%d")) {
     name = g_strdup_printf ("video_%d", mux->num_v_streams++);
-    setcapsfunc = gst_matroska_mux_video_pad_setcaps;
+    setcapsfunc = GST_DEBUG_FUNCPTR (gst_matroska_mux_video_pad_setcaps);
     context = (GstMatroskaTrackContext *)
         g_new0 (GstMatroskaTrackVideoContext, 1);
     context->type = GST_MATROSKA_TRACK_TYPE_VIDEO;
     context->name = g_strdup ("Video");
   } else if (templ == gst_element_class_get_pad_template (klass, "subtitle_%d")) {
     name = g_strdup_printf ("subtitle_%d", mux->num_t_streams++);
-    setcapsfunc = gst_matroska_mux_subtitle_pad_setcaps;
+    setcapsfunc = GST_DEBUG_FUNCPTR (gst_matroska_mux_subtitle_pad_setcaps);
     context = (GstMatroskaTrackContext *)
         g_new0 (GstMatroskaTrackSubtitleContext, 1);
     context->type = GST_MATROSKA_TRACK_TYPE_SUBTITLE;
@@ -1133,9 +1190,13 @@ gst_matroska_mux_request_new_pad (GstElement * element,
   g_free (name);
   collect_pad = (GstMatroskaPad *)
       gst_collect_pads_add_pad (mux->collect, newpad, sizeof (GstMatroskaPad));
+
+  /* TODO: check default values for the context */
   context->flags = GST_MATROSKA_TRACK_ENABLED | GST_MATROSKA_TRACK_DEFAULT;
   collect_pad->track = context;
   collect_pad->buffer = NULL;
+  collect_pad->start_ts = GST_CLOCK_TIME_NONE;
+  collect_pad->end_ts = GST_CLOCK_TIME_NONE;
 
   /* FIXME: hacked way to override/extend the event function of
    * GstCollectPads; because it sets its own event function giving the
@@ -1152,6 +1213,7 @@ gst_matroska_mux_request_new_pad (GstElement * element,
   gst_pad_set_setcaps_function (newpad, setcapsfunc);
   gst_pad_set_active (newpad, TRUE);
   gst_element_add_pad (element, newpad);
+  mux->num_streams++;
 
   return newpad;
 }
@@ -1167,20 +1229,34 @@ static void
 gst_matroska_mux_release_pad (GstElement * element, GstPad * pad)
 {
   GstMatroskaMux *mux;
+
   GSList *walk;
 
   mux = GST_MATROSKA_MUX (GST_PAD_PARENT (pad));
 
   for (walk = mux->collect->data; walk; walk = g_slist_next (walk)) {
     GstCollectData *cdata = (GstCollectData *) walk->data;
+
     GstMatroskaPad *collect_pad = (GstMatroskaPad *) cdata;
 
     if (cdata->pad == pad) {
-      if (collect_pad->duration > mux->duration)
+      GstClockTime min_dur;     /* observed minimum duration */
+
+      if (GST_CLOCK_TIME_IS_VALID (collect_pad->start_ts) &&
+          GST_CLOCK_TIME_IS_VALID (collect_pad->end_ts)) {
+        min_dur = GST_CLOCK_DIFF (collect_pad->start_ts, collect_pad->end_ts);
+        if (collect_pad->duration < min_dur)
+          collect_pad->duration = min_dur;
+      }
+
+      if (GST_CLOCK_TIME_IS_VALID (collect_pad->duration) &&
+          mux->duration < collect_pad->duration)
         mux->duration = collect_pad->duration;
+
       gst_matroska_pad_free (collect_pad);
       gst_collect_pads_remove_pad (mux->collect, pad);
       gst_element_remove_pad (element, pad);
+      mux->num_streams--;
       return;
     }
   }
@@ -1201,8 +1277,11 @@ gst_matroska_mux_track_header (GstMatroskaMux * mux,
     GstMatroskaTrackContext * context)
 {
   GstEbmlWrite *ebml = mux->ebml_write;
+
   guint64 master;
 
+  /* TODO: check if everything necessary is written and check default values */
+
   /* track type goes before the type-specific stuff */
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKNUMBER, context->num);
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKTYPE, context->type);
@@ -1280,6 +1359,7 @@ gst_matroska_mux_track_header (GstMatroskaMux * mux,
   /* FIXME: until we have a nice way of getting the codecname
    * out of the caps, I'm not going to enable this. Too much
    * (useless, double, boring) work... */
+  /* TODO: Use value from tags if any */
   /*gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_CODECNAME,
      context->codec_name); */
   gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_TRACKNAME, context->name);
@@ -1296,19 +1376,25 @@ static void
 gst_matroska_mux_start (GstMatroskaMux * mux)
 {
   GstEbmlWrite *ebml = mux->ebml_write;
-  guint32 seekhead_id[] = { GST_MATROSKA_ID_INFO,
+
+  guint32 seekhead_id[] = { GST_MATROSKA_ID_SEGMENTINFO,
     GST_MATROSKA_ID_TRACKS,
     GST_MATROSKA_ID_CUES,
-    GST_MATROSKA_ID_SEEKHEAD,
     GST_MATROSKA_ID_TAGS,
     0
   };
   guint64 master, child;
+
   GSList *collected;
+
   int i;
+
   guint tracknum = 1;
-  gdouble duration = 0;
+
+  GstClockTime duration = 0;
+
   guint32 *segment_uid = (guint32 *) g_malloc (16);
+
   GRand *rand = g_rand_new ();
   GTimeVal time = { 0, 0 };
 
@@ -1336,7 +1422,7 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
 
   /* segment info */
   mux->info_pos = ebml->pos;
-  master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_INFO);
+  master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_SEGMENTINFO);
   for (i = 0; i < 4; i++) {
     segment_uid[i] = g_rand_int (rand);
   }
@@ -1350,27 +1436,30 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   for (collected = mux->collect->data; collected;
       collected = g_slist_next (collected)) {
     GstMatroskaPad *collect_pad;
+
     GstFormat format = GST_FORMAT_TIME;
-    GstPad *thepad, *peerpad;
+
+    GstPad *thepad;
+
     gint64 trackduration;
 
     collect_pad = (GstMatroskaPad *) collected->data;
     thepad = collect_pad->collect.pad;
 
     /* Query the total length of the track. */
-    peerpad = gst_pad_get_peer (thepad);
-    GST_DEBUG_OBJECT (thepad, "querying duration");
-    if (gst_pad_query_duration (peerpad, &format, &trackduration)) {
+    GST_DEBUG_OBJECT (thepad, "querying peer duration");
+    if (gst_pad_query_peer_duration (thepad, &format, &trackduration)) {
       GST_DEBUG_OBJECT (thepad, "duration: %" GST_TIME_FORMAT,
           GST_TIME_ARGS (trackduration));
-      if ((gdouble) trackduration > duration) {
-        duration = (gdouble) trackduration;
+      if (trackduration != GST_CLOCK_TIME_NONE && trackduration > duration) {
+        duration = (GstClockTime) trackduration;
       }
     }
-    gst_object_unref (peerpad);
   }
   gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
-      duration / gst_guint64_to_gdouble (mux->time_scale));
+      gst_guint64_to_gdouble (duration) /
+      gst_guint64_to_gdouble (mux->time_scale));
+
   gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_MUXINGAPP,
       "GStreamer plugin version " PACKAGE_VERSION);
   if (mux->writing_app && mux->writing_app[0]) {
@@ -1388,6 +1477,7 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
       collected = g_slist_next (collected)) {
 
     GstMatroskaPad *collect_pad;
+
     GstPad *thepad;
 
     collect_pad = (GstMatroskaPad *) collected->data;
@@ -1411,6 +1501,7 @@ static void
 gst_matroska_mux_write_simple_tag (const GstTagList * list, const gchar * tag,
     gpointer data)
 {
+  /* TODO: more sensible tag mappings */
   struct
   {
     gchar *matroska_tagname;
@@ -1423,35 +1514,46 @@ gst_matroska_mux_write_simple_tag (const GstTagList * list, const gchar * tag,
     GST_MATROSKA_TAG_ID_ALBUM, GST_TAG_ALBUM}, {
     GST_MATROSKA_TAG_ID_COMMENTS, GST_TAG_COMMENT}, {
     GST_MATROSKA_TAG_ID_BITSPS, GST_TAG_BITRATE}, {
+    GST_MATROSKA_TAG_ID_BPS, GST_TAG_BITRATE}, {
+    GST_MATROSKA_TAG_ID_ENCODER, GST_TAG_ENCODER}, {
     GST_MATROSKA_TAG_ID_DATE, GST_TAG_DATE}, {
     GST_MATROSKA_TAG_ID_ISRC, GST_TAG_ISRC}, {
-    GST_MATROSKA_TAG_ID_COPYRIGHT, GST_TAG_COPYRIGHT}
+    GST_MATROSKA_TAG_ID_COPYRIGHT, GST_TAG_COPYRIGHT}, {
+    GST_MATROSKA_TAG_ID_BPM, GST_TAG_BEATS_PER_MINUTE}, {
+    GST_MATROSKA_TAG_ID_TERMS_OF_USE, GST_TAG_LICENSE}, {
+    GST_MATROSKA_TAG_ID_COMPOSER, GST_TAG_COMPOSER}, {
+    GST_MATROSKA_TAG_ID_LEAD_PERFORMER, GST_TAG_PERFORMER}, {
+    GST_MATROSKA_TAG_ID_GENRE, GST_TAG_GENRE}
   };
+
   GstEbmlWrite *ebml = (GstEbmlWrite *) data;
+
   guint i;
+
   guint64 simpletag_master;
 
   for (i = 0; i < G_N_ELEMENTS (tag_conv); i++) {
     const gchar *tagname_gst = tag_conv[i].gstreamer_tagname;
+
     const gchar *tagname_mkv = tag_conv[i].matroska_tagname;
 
     if (strcmp (tagname_gst, tag) == 0) {
       GValue src = { 0, };
-      GValue dest = { 0, };
+      gchar *dest;
 
       if (!gst_tag_list_copy_value (&src, list, tag))
         break;
-      g_value_init (&dest, G_TYPE_STRING);
-      g_value_transform (&src, &dest);
-      g_value_unset (&src);
+      if ((dest = gst_value_serialize (&src))) {
 
-      simpletag_master = gst_ebml_write_master_start (ebml,
-          GST_MATROSKA_ID_SIMPLETAG);
-      gst_ebml_write_ascii (ebml, GST_MATROSKA_ID_TAGNAME, tagname_mkv);
-      gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_TAGSTRING,
-          g_value_get_string (&dest));
-      gst_ebml_write_master_finish (ebml, simpletag_master);
-      g_value_unset (&dest);
+        simpletag_master = gst_ebml_write_master_start (ebml,
+            GST_MATROSKA_ID_SIMPLETAG);
+        gst_ebml_write_ascii (ebml, GST_MATROSKA_ID_TAGNAME, tagname_mkv);
+        gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_TAGSTRING, dest);
+        gst_ebml_write_master_finish (ebml, simpletag_master);
+      } else {
+        GST_WARNING ("Can't transform tag '%s' to string", tagname_mkv);
+      }
+      g_value_unset (&src);
       break;
     }
   }
@@ -1468,9 +1570,13 @@ static void
 gst_matroska_mux_finish (GstMatroskaMux * mux)
 {
   GstEbmlWrite *ebml = mux->ebml_write;
+
   guint64 pos;
+
   guint64 duration = 0;
+
   GSList *collected;
+
   GstTagList *tags;
 
   /* finish last cluster */
@@ -1481,6 +1587,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   /* cues */
   if (mux->index != NULL) {
     guint n;
+
     guint64 master, pointentry_master, trackpos_master;
 
     mux->cues_pos = ebml->pos;
@@ -1495,7 +1602,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CUETIME,
           idx->time / mux->time_scale);
       trackpos_master = gst_ebml_write_master_start (ebml,
-          GST_MATROSKA_ID_CUETRACKPOSITION);
+          GST_MATROSKA_ID_CUETRACKPOSITIONS);
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CUETRACK, idx->track);
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CUECLUSTERPOSITION,
           idx->pos - mux->segment_master);
@@ -1507,28 +1614,6 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_flush_cache (ebml);
   }
 
-  if (mux->meta_index != NULL) {
-    guint n;
-    guint64 master, seekentry_master;
-
-    mux->meta_pos = ebml->pos;
-    gst_ebml_write_set_cache (ebml, 12 + 28 * mux->num_meta_indexes);
-    master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_SEEKHEAD);
-
-    for (n = 0; n < mux->num_meta_indexes; n++) {
-      GstMatroskaMetaSeekIndex *idx = &mux->meta_index[n];
-
-      seekentry_master = gst_ebml_write_master_start (ebml,
-          GST_MATROSKA_ID_SEEKENTRY);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKID, idx->id);
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKPOSITION,
-          idx->pos - mux->segment_master);
-      gst_ebml_write_master_finish (ebml, seekentry_master);
-    }
-    gst_ebml_write_master_finish (ebml, master);
-  }
-  gst_ebml_write_flush_cache (ebml);
-
   /* tags */
   tags = gst_tag_list_merge (gst_tag_setter_get_tag_list (GST_TAG_SETTER (mux)),
       mux->tags, GST_TAG_MERGE_APPEND);
@@ -1536,6 +1621,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
   if (tags != NULL) {
     guint64 master_tags, master_tag;
 
+    /* TODO: maybe limit via the TARGETS id by looking at the source pad */
     mux->tags_pos = ebml->pos;
     master_tags = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TAGS);
     master_tag = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TAG);
@@ -1570,25 +1656,14 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
     gst_ebml_write_seek (ebml, my_pos);
   }
-  if (mux->meta_index != NULL) {
-    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
-        mux->meta_pos - mux->segment_master);
-  } else {
-    /* void'ify */
-    guint64 my_pos = ebml->pos;
-
-    gst_ebml_write_seek (ebml, mux->seekhead_pos + 96);
-    gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
-    gst_ebml_write_seek (ebml, my_pos);
-  }
   if (tags != NULL) {
-    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 144,
+    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
         mux->tags_pos - mux->segment_master);
   } else {
     /* void'ify */
     guint64 my_pos = ebml->pos;
 
-    gst_ebml_write_seek (ebml, mux->seekhead_pos + 124);
+    gst_ebml_write_seek (ebml, mux->seekhead_pos + 96);
     gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
     gst_ebml_write_seek (ebml, my_pos);
   }
@@ -1601,17 +1676,45 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
       collected = g_slist_next (collected)) {
     GstMatroskaPad *collect_pad;
 
+    GstClockTime min_duration;  /* observed minimum duration */
+
     collect_pad = (GstMatroskaPad *) collected->data;
 
-    if (collect_pad->duration > duration)
+    GST_DEBUG_OBJECT (mux, "Pad %" GST_PTR_FORMAT " start ts %" GST_TIME_FORMAT
+        " end ts %" GST_TIME_FORMAT, collect_pad,
+        GST_TIME_ARGS (collect_pad->start_ts),
+        GST_TIME_ARGS (collect_pad->end_ts));
+
+    if (GST_CLOCK_TIME_IS_VALID (collect_pad->start_ts) &&
+        GST_CLOCK_TIME_IS_VALID (collect_pad->end_ts)) {
+      min_duration =
+          GST_CLOCK_DIFF (collect_pad->start_ts, collect_pad->end_ts);
+      if (collect_pad->duration < min_duration)
+        collect_pad->duration = min_duration;
+      GST_DEBUG_OBJECT (collect_pad, "final track duration: %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (collect_pad->duration));
+    }
+
+    if (GST_CLOCK_TIME_IS_VALID (collect_pad->duration) &&
+        duration < collect_pad->duration)
       duration = collect_pad->duration;
   }
   if (duration != 0) {
+    GST_DEBUG_OBJECT (mux, "final total duration: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (duration));
     pos = mux->ebml_write->pos;
     gst_ebml_write_seek (ebml, mux->duration_pos);
     gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
-        gst_guint64_to_gdouble (duration / mux->time_scale));
+        gst_guint64_to_gdouble (duration) /
+        gst_guint64_to_gdouble (mux->time_scale));
     gst_ebml_write_seek (ebml, pos);
+  } else {
+    /* void'ify */
+    guint64 my_pos = ebml->pos;
+
+    gst_ebml_write_seek (ebml, mux->duration_pos);
+    gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 8);
+    gst_ebml_write_seek (ebml, my_pos);
   }
 
   /* finish segment - this also writes element length */
@@ -1633,6 +1736,7 @@ static GstMatroskaPad *
 gst_matroska_mux_best_pad (GstMatroskaMux * mux, gboolean * popped)
 {
   GSList *collected;
+
   GstMatroskaPad *best = NULL;
 
   *popped = FALSE;
@@ -1705,12 +1809,19 @@ static GstFlowReturn
 gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
 {
   GstEbmlWrite *ebml = mux->ebml_write;
+
   GstBuffer *buf, *hdr;
+
   guint64 cluster, blockgroup;
+
   gboolean write_duration;
+
   gint16 relative_timestamp;
+
   gint64 relative_timestamp64;
+
   guint64 block_duration;
+
   gboolean is_video_keyframe = FALSE;
 
   /* write data */
@@ -1727,6 +1838,8 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
 
   /* hm, invalid timestamp (due to --to be fixed--- element upstream);
    * this would wreak havoc with time stored in matroska file */
+  /* TODO: maybe calculate a timestamp by using the previous timestamp
+   * and default duration */
   if (!GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
     GST_WARNING_OBJECT (collect_pad->collect.pad,
         "Invalid buffer timestamp; dropping buffer");
@@ -1748,7 +1861,6 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
     /* start a new cluster every two seconds or at keyframe */
     if (mux->cluster_time + GST_SECOND * 2 < GST_BUFFER_TIMESTAMP (buf)
         || is_video_keyframe) {
-      GstMatroskaMetaSeekIndex *idx;
 
       gst_ebml_write_master_finish (ebml, mux->cluster);
       mux->cluster_pos = ebml->pos;
@@ -1757,32 +1869,15 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
           GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
       mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
-
-      if (mux->num_meta_indexes % 32 == 0) {
-        mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
-            mux->num_meta_indexes + 32);
-      }
-      idx = &mux->meta_index[mux->num_meta_indexes++];
-      idx->id = GST_MATROSKA_ID_CLUSTER;
-      idx->pos = mux->cluster_pos;
     }
   } else {
     /* first cluster */
-    GstMatroskaMetaSeekIndex *idx;
 
     mux->cluster_pos = ebml->pos;
     mux->cluster = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
     gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
         GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
     mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
-
-    if (mux->num_meta_indexes % 32 == 0) {
-      mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
-          mux->num_meta_indexes + 32);
-    }
-    idx = &mux->meta_index[mux->num_meta_indexes++];
-    idx->id = GST_MATROSKA_ID_CLUSTER;
-    idx->pos = mux->cluster_pos;
   }
   cluster = mux->cluster;
 
@@ -1795,6 +1890,10 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
    * for audio only files. This can be largely improved, such as doing
    * one for each keyframe or each second (for all-keyframe
    * streams), only the *first* video track. But that'll come later... */
+
+  /* TODO: index is useful for every track, should contain the number of
+   * the block in the cluster which contains the timestamp
+   */
   if (is_video_keyframe) {
     GstMatroskaIndex *idx;
 
@@ -1833,7 +1932,7 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux, GstMatroskaPad * collect_pad)
 
   /* write the block, for matroska v2 use SimpleBlock if possible
    * one slice (*breath*).
-   * FIXME: lacing, etc. */
+   * FIXME: Need to do correct lacing! */
   relative_timestamp64 = GST_BUFFER_TIMESTAMP (buf) - mux->cluster_time;
   if (relative_timestamp64 >= 0) {
     /* round the timestamp */
@@ -1888,8 +1987,11 @@ static GstFlowReturn
 gst_matroska_mux_collected (GstCollectPads * pads, gpointer user_data)
 {
   GstMatroskaMux *mux = GST_MATROSKA_MUX (user_data);
+
   GstMatroskaPad *best;
+
   gboolean popped;
+
   GstFlowReturn ret;
 
   GST_DEBUG_OBJECT (mux, "Collected pads");
@@ -1918,7 +2020,30 @@ gst_matroska_mux_collected (GstCollectPads * pads, gpointer user_data)
       ret = GST_FLOW_UNEXPECTED;
       break;
     }
-    GST_DEBUG_OBJECT (best->collect.pad, "best pad");
+    GST_DEBUG_OBJECT (best->collect.pad, "best pad - buffer ts %"
+        GST_TIME_FORMAT " dur %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (best->buffer)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (best->buffer)));
+
+    /* make note of first and last encountered timestamps, so we can calculate
+     * the actual duration later when we send an updated header on eos */
+    if (GST_BUFFER_TIMESTAMP_IS_VALID (best->buffer)) {
+      GstClockTime start_ts = GST_BUFFER_TIMESTAMP (best->buffer);
+
+      GstClockTime end_ts = start_ts;
+
+      if (GST_BUFFER_DURATION_IS_VALID (best->buffer))
+        end_ts += GST_BUFFER_DURATION (best->buffer);
+      else if (best->track->default_duration)
+        end_ts += best->track->default_duration;
+
+      if (!GST_CLOCK_TIME_IS_VALID (best->end_ts) || end_ts > best->end_ts)
+        best->end_ts = end_ts;
+
+      if (G_UNLIKELY (best->start_ts == GST_CLOCK_TIME_NONE ||
+              start_ts < best->start_ts))
+        best->start_ts = start_ts;
+    }
 
     /* write one buffer */
     ret = gst_matroska_mux_write_data (mux, best);
@@ -1941,6 +2066,7 @@ static GstStateChangeReturn
 gst_matroska_mux_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
+
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
 
   switch (transition) {
@@ -1991,7 +2117,7 @@ gst_matroska_mux_set_property (GObject * object,
         break;
       }
       g_free (mux->writing_app);
-      mux->writing_app = g_strdup (g_value_get_string (value));
+      mux->writing_app = g_value_dup_string (value);
       break;
     case ARG_MATROSKA_VERSION:
       mux->matroska_version = g_value_get_int (value);
