@@ -76,6 +76,8 @@ typedef struct
   GstBuffer *buffer;            /* the queued buffer for this pad */
 
   guint64 duration;
+  GstClockTime start_ts;
+  GstClockTime end_ts;    /* last timestamp + (if available) duration */
 }
 GstMatroskaPad;
 
@@ -127,10 +129,6 @@ typedef struct _GstMatroskaMux {
                  cluster_time,
                  cluster_pos;
 
-  /* meta-seek info */
-  GstMatroskaMetaSeekIndex *meta_index;
-  guint          num_meta_indexes;
-
   /* tags */
   GstTagList     *tags;
 } GstMatroskaMux;
