@@ -28,6 +28,9 @@
 
 #include "gstrtpgsmpay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpgsmpay_debug);
+#define GST_CAT_DEFAULT (rtpgsmpay_debug)
+
 /* elementfactory information */
 static const GstElementDetails gst_rtp_gsm_pay_details =
 GST_ELEMENT_DETAILS ("RTP GSM audio payloader",
@@ -43,12 +46,16 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 static GstStaticPadTemplate gst_rtp_gsm_pay_src_template =
-GST_STATIC_PAD_TEMPLATE ("src",
+    GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"audio\", "
         "payload = (int) " GST_RTP_PAYLOAD_GSM_STRING ", "
+        "clock-rate = (int) 8000, " "encoding-name = (string) \"GSM\"; "
+        "application/x-rtp, "
+        "media = (string) \"audio\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) 8000, " "encoding-name = (string) \"GSM\"")
     );
 
@@ -87,6 +94,9 @@ gst_rtp_gsm_pay_class_init (GstRTPGSMPayClass * klass)
 
   gstbasertppayload_class->set_caps = gst_rtp_gsm_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_gsm_pay_handle_buffer;
+
+  GST_DEBUG_CATEGORY_INIT (rtpgsmpay_debug, "rtpgsmpay", 0,
+      "GSM Audio RTP Payloader");
 }
 
 static void
@@ -125,13 +135,14 @@ gst_rtp_gsm_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   guint size, payload_len;
   GstBuffer *outbuf;
   guint8 *payload, *data;
-  GstClockTime timestamp;
+  GstClockTime timestamp, duration;
   GstFlowReturn ret;
 
   rtpgsmpay = GST_RTP_GSM_PAY (basepayload);
 
   size = GST_BUFFER_SIZE (buffer);
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
 
   /* FIXME, only one GSM frame per RTP packet for now */
   payload_len = size;
@@ -140,8 +151,9 @@ gst_rtp_gsm_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   /* FIXME, assert for now */
   g_assert (payload_len <= GST_BASE_RTP_PAYLOAD_MTU (rtpgsmpay));
 
-  /* copy timestamp */
+  /* copy timestamp and duration */
   GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
+  GST_BUFFER_DURATION (outbuf) = duration;
 
   /* get payload */
   payload = gst_rtp_buffer_get_payload (outbuf);
