@@ -37,18 +37,6 @@ GST_ELEMENT_DETAILS ("RTP packet depayloader",
     "Extracts Theora video from RTP packets (draft-01 of RFC XXXX)",
     "Wim Taymans <wim@fluendo.com>");
 
-/* RtpTheoraDepay signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0,
-};
-
 static GstStaticPadTemplate gst_rtp_theora_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -56,7 +44,9 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"THEORA\""
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\","
+        /* only support inline delivery */
+        "delivery-method = (string) \"inline\""
         /* All required parameters 
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
@@ -87,10 +77,6 @@ static gboolean gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_theora_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_theora_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
 static void gst_rtp_theora_depay_finalize (GObject * object);
 
 static GstStateChangeReturn gst_rtp_theora_depay_change_state (GstElement *
@@ -121,8 +107,6 @@ gst_rtp_theora_depay_class_init (GstRtpTheoraDepayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  gobject_class->set_property = gst_rtp_theora_depay_set_property;
-  gobject_class->get_property = gst_rtp_theora_depay_get_property;
   gobject_class->finalize = gst_rtp_theora_depay_finalize;
 
   gstelement_class->change_state = gst_rtp_theora_depay_change_state;
@@ -347,25 +331,21 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   GstCaps *srccaps;
   const gchar *delivery_method;
   const gchar *configuration;
-  gint clock_rate;
 
   rtptheoradepay = GST_RTP_THEORA_DEPAY (depayload);
 
   structure = gst_caps_get_structure (caps, 0);
 
-  /* get clockrate */
-  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
-    goto no_rate;
-
   /* see how the configuration parameters will be transmitted */
   delivery_method = gst_structure_get_string (structure, "delivery-method");
   if (delivery_method == NULL)
     goto no_delivery_method;
 
-  if (g_strcasecmp (delivery_method, "inline")) {
+  if (!g_ascii_strcasecmp (delivery_method, "inline")) {
     /* configure string is in the caps */
-  } else if (g_strcasecmp (delivery_method, "in_band")) {
+  } else if (!g_ascii_strcasecmp (delivery_method, "in_band")) {
     /* headers will (also) be transmitted in the RTP packets */
+    goto unsupported_delivery_method;
   } else if (g_str_has_prefix (delivery_method, "out_band/")) {
     /* some other method of header delivery. */
     goto unsupported_delivery_method;
@@ -380,14 +360,14 @@ gst_rtp_theora_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if (!gst_rtp_theora_depay_parse_configuration (rtptheoradepay, configuration))
     goto invalid_configuration;
 
-  /* caps seem good, configure element */
-  depayload->clock_rate = clock_rate;
-
   /* set caps on pad and on header */
   srccaps = gst_caps_new_simple ("video/x-theora", NULL);
   gst_pad_set_caps (depayload->srcpad, srccaps);
   gst_caps_unref (srccaps);
 
+  /* Clock rate is always 90000 according to draft-barbato-avt-rtp-theora-01 */
+  depayload->clock_rate = 90000;
+
   return TRUE;
 
   /* ERRORS */
@@ -412,11 +392,6 @@ invalid_configuration:
     GST_ERROR_OBJECT (rtptheoradepay, "invalid configuration specified");
     return FALSE;
   }
-no_rate:
-  {
-    GST_ERROR_OBJECT (rtptheoradepay, "no clock-rate specified");
-    return FALSE;
-  }
 }
 
 static gboolean
@@ -503,6 +478,9 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   F = (header & 0xc0) >> 6;
   packets = (header & 0xf);
 
+  GST_DEBUG_OBJECT (depayload, "ident: 0x%08x, F: %d, TDT: %d, packets: %d",
+      ident, F, TDT, packets);
+
   if (TDT == 0) {
     gboolean do_switch = FALSE;
 
@@ -525,9 +503,6 @@ gst_rtp_theora_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   payload += 4;
   payload_len -= 4;
 
-  GST_DEBUG_OBJECT (depayload, "ident: %u, F: %d, TDT: %d, packets: %d", ident,
-      F, TDT, packets);
-
   /* fragmented packets, assemble */
   if (F != 0) {
     GstBuffer *vdata;
@@ -670,36 +645,6 @@ length_short:
   }
 }
 
-static void
-gst_rtp_theora_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraDepay *rtptheoradepay;
-
-  rtptheoradepay = GST_RTP_THEORA_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_theora_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraDepay *rtptheoradepay;
-
-  rtptheoradepay = GST_RTP_THEORA_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_rtp_theora_depay_change_state (GstElement * element,
     GstStateChange transition)
