@@ -51,7 +51,8 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"THEORA\""
+        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\", "
+        "delivery-method = (string) \"inline\""
         /* All required parameters
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
@@ -79,6 +80,8 @@ GST_BOILERPLATE (GstRtpTheoraPay, gst_rtp_theora_pay, GstBaseRTPPayload,
 
 static gboolean gst_rtp_theora_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement *
+    element, GstStateChange transition);
 static GstFlowReturn gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
 
@@ -99,7 +102,9 @@ static void
 gst_rtp_theora_pay_class_init (GstRtpTheoraPayClass * klass)
 {
   GObjectClass *gobject_class;
+
   GstElementClass *gstelement_class;
+
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
   gobject_class = (GObjectClass *) klass;
@@ -108,6 +113,8 @@ gst_rtp_theora_pay_class_init (GstRtpTheoraPayClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  gstelement_class->change_state = gst_rtp_theora_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_theora_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_theora_pay_handle_buffer;
 
@@ -119,6 +126,19 @@ static void
 gst_rtp_theora_pay_init (GstRtpTheoraPay * rtptheorapay,
     GstRtpTheoraPayClass * klass)
 {
+  /* needed because of GST_BOILERPLATE */
+}
+
+static void
+gst_rtp_theora_pay_cleanup (GstRtpTheoraPay * rtptheorapay)
+{
+  g_list_foreach (rtptheorapay->headers, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (rtptheorapay->headers);
+  rtptheorapay->headers = NULL;
+
+  if (rtptheorapay->packet)
+    gst_buffer_unref (rtptheorapay->packet);
+  rtptheorapay->packet = NULL;
 }
 
 static gboolean
@@ -163,6 +183,7 @@ gst_rtp_theora_pay_init_packet (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
       gst_rtp_buffer_new_allocate_len (GST_BASE_RTP_PAYLOAD_MTU
       (rtptheorapay), 0, 0);
   gst_rtp_theora_pay_reset_packet (rtptheorapay, TDT);
+
   GST_BUFFER_TIMESTAMP (rtptheorapay->packet) = timestamp;
 }
 
@@ -170,7 +191,9 @@ static GstFlowReturn
 gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
 {
   GstFlowReturn ret;
+
   guint8 *payload;
+
   guint hlen;
 
   /* check for empty packet */
@@ -203,6 +226,8 @@ gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
   hlen = gst_rtp_buffer_calc_header_len (0);
   GST_BUFFER_SIZE (rtptheorapay->packet) = hlen + rtptheorapay->payload_pos;
 
+  GST_BUFFER_DURATION (rtptheorapay->packet) = rtptheorapay->payload_duration;
+
   /* push, this gives away our ref to the packet, so clear it. */
   ret =
       gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtptheorapay),
@@ -216,6 +241,7 @@ static gchar *
 encode_base64 (const guint8 * in, guint size, guint * len)
 {
   gchar *ret, *d;
+
   static const gchar *v =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
@@ -241,16 +267,23 @@ static gboolean
 gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
   GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
+
   GList *walk;
+
   guint length, size, n_headers, configlen;
+
   gchar *wstr, *hstr, *configuration;
+
   guint8 *data, *config;
+
   guint32 ident;
 
   GST_DEBUG_OBJECT (rtptheorapay, "finish headers");
 
-  if (!rtptheorapay->headers)
+  if (!rtptheorapay->headers) {
+    GST_DEBUG_OBJECT (rtptheorapay, "We need 2 headers but have none");
     goto no_headers;
+  }
 
   /* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                     Number of packed headers                  |
@@ -302,6 +335,7 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   ident = fnv1_hash_32_new ();
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+
     guint bsize;
 
     bsize = GST_BUFFER_SIZE (buf);
@@ -351,6 +385,7 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   /* store length for each header */
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
+
     guint bsize, size, temp;
 
     /* only need to store the length when it's not the last header */
@@ -421,6 +456,7 @@ gst_rtp_theora_pay_parse_id (GstBaseRTPPayload * basepayload, guint8 * data,
     guint size)
 {
   GstRtpTheoraPay *rtptheorapay;
+
   gint width, height;
 
   rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
@@ -478,15 +514,25 @@ gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
     GstBuffer * buffer)
 {
   GstRtpTheoraPay *rtptheorapay;
+
   GstFlowReturn ret;
+
   guint size, newsize;
+
   guint8 *data;
+
   guint packet_len;
+
   GstClockTime duration, newduration, timestamp;
+
   gboolean flush;
+
   guint8 TDT;
+
   guint plen;
+
   guint8 *ppos, *payload;
+
   gboolean fragmented;
 
   rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
@@ -620,6 +666,8 @@ gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
         rtptheorapay->payload_duration += duration;
     }
   }
+  gst_buffer_unref (buffer);
+
 done:
   return ret;
 
@@ -628,26 +676,57 @@ wrong_size:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         ("Invalid packet size (1 < %d <= 0xffff)", size), (NULL));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 parse_id_failed:
   {
+    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 unknown_header:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Ignoring unknown header received"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 header_error:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Error initializing header config"));
+    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 }
 
+static GstStateChangeReturn
+gst_rtp_theora_pay_change_state (GstElement * element,
+    GstStateChange transition)
+{
+  GstRtpTheoraPay *rtptheorapay;
+
+  GstStateChangeReturn ret;
+
+  rtptheorapay = GST_RTP_THEORA_PAY (element);
+
+  switch (transition) {
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_rtp_theora_pay_cleanup (rtptheorapay);
+      break;
+    default:
+      break;
+  }
+  return ret;
+}
+
 gboolean
 gst_rtp_theora_pay_plugin_init (GstPlugin * plugin)
 {
