@@ -48,8 +48,11 @@
 
 G_BEGIN_DECLS
 
-#include "gstrtsp.h"
-#include "rtsp.h"
+#include <gst/rtsp/gstrtspconnection.h>
+#include <gst/rtsp/gstrtspmessage.h>
+#include <gst/rtsp/gstrtspurl.h>
+
+#include "gstrtspext.h"
 
 #define GST_TYPE_RTSPSRC \
   (gst_rtspsrc_get_type())
@@ -71,14 +74,16 @@ typedef struct _GstRTSPSrcClass GstRTSPSrcClass;
 #define GST_RTSP_STATE_LOCK(rtsp)        (g_static_rec_mutex_lock (GST_RTSP_STATE_GET_LOCK(rtsp)))
 #define GST_RTSP_STATE_UNLOCK(rtsp)      (g_static_rec_mutex_unlock (GST_RTSP_STATE_GET_LOCK(rtsp)))
 
-#define GST_RTSP_STREAM_GET_LOCK(rtsp)    (GST_RTSPSRC_CAST(rtsp)->stream_rec_lock)
-#define GST_RTSP_STREAM_LOCK(rtsp)        (g_static_rec_mutex_lock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
-#define GST_RTSP_STREAM_UNLOCK(rtsp)      (g_static_rec_mutex_unlock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
+#define GST_RTSP_STREAM_GET_LOCK(rtsp)   (GST_RTSPSRC_CAST(rtsp)->stream_rec_lock)
+#define GST_RTSP_STREAM_LOCK(rtsp)       (g_static_rec_mutex_lock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
+#define GST_RTSP_STREAM_UNLOCK(rtsp)     (g_static_rec_mutex_unlock (GST_RTSP_STREAM_GET_LOCK(rtsp)))
+
+#define GST_RTSP_CONN_GET_LOCK(rtsp)     (GST_RTSPSRC_CAST(rtsp)->conn_rec_lock)
+#define GST_RTSP_CONN_LOCK(rtsp)         (g_static_rec_mutex_lock (GST_RTSP_CONN_GET_LOCK(rtsp)))
+#define GST_RTSP_CONN_UNLOCK(rtsp)       (g_static_rec_mutex_unlock (GST_RTSP_CONN_GET_LOCK(rtsp)))
 
 typedef struct _GstRTSPStream GstRTSPStream;
 
-#include "rtspext.h"
-
 struct _GstRTSPStream {
   gint          id;
 
@@ -88,6 +93,9 @@ struct _GstRTSPStream {
   GstPad       *srcpad;
   GstFlowReturn last_ret;
   gboolean      added;
+  gboolean      disabled;
+  gboolean      eos;
+  gboolean      discont;
 
   /* for interleaved mode */
   guint8        channel[2];
@@ -112,6 +120,11 @@ struct _GstRTSPStream {
   guint32       ssrc; 
   guint32       seqbase;
   guint64       timebase;
+
+  /* bandwidth */
+  guint         as_bandwidth;
+  guint         rs_bandwidth;
+  guint         rr_bandwidth;
 };
 
 struct _GstRTSPSrc {
@@ -123,10 +136,11 @@ struct _GstRTSPSrc {
   GStaticRecMutex *stream_rec_lock;
   GstSegment       segment;
   gboolean         running;
-  gdouble          position;
+  gboolean         need_range;
   gint             free_channel;
   GstEvent        *close_segment;
   GstEvent        *start_segment;
+  GstClockTime     base_time;
 
   /* UDP mode loop */
   gint             loop_cmd;
@@ -135,40 +149,47 @@ struct _GstRTSPSrc {
   /* mutex for protecting state changes */
   GStaticRecMutex *state_rec_lock;
 
+  /* mutex for protecting the connection */
+  GStaticRecMutex *conn_rec_lock;
+
   gint             numstreams;
   GList           *streams;
   GstStructure    *props;
   gboolean         need_activate;
 
   /* properties */
-  gchar           *location;
-  gchar           *req_location; /* Sanitised URL to use in network requests */
-  RTSPUrl         *url;
-  RTSPLowerTrans   protocols;
-  gboolean         debug;
-  guint   	   retry;
-  guint64          udp_timeout;
-  GTimeVal         tcp_timeout;
-  guint            latency;
+  gchar            *location;
+  gchar            *req_location; /* Sanitised URL to use in network requests */
+  GstRTSPUrl       *url;
+  GstRTSPLowerTrans protocols;
+  gboolean          debug;
+  guint   	    retry;
+  guint64           udp_timeout;
+  GTimeVal          tcp_timeout;
+  GTimeVal         *ptcp_timeout;
+  guint             latency;
+  guint             connection_speed;
 
   /* state */
-  RTSPState        state;
-  gchar           *content_base;
-  RTSPLowerTrans   cur_protocols;
-  gboolean         tried_url_auth;
-  gchar           *addr;
+  GstRTSPState       state;
+  gchar             *content_base;
+  GstRTSPLowerTrans  cur_protocols;
+  gboolean           tried_url_auth;
+  gchar             *addr;
+  gboolean           need_redirect;
 
   /* supported methods */
-  gint             methods;
+  gint               methods;
 
   /* session management */
   GstElement      *session;
   gulong           session_sig_id;
   gulong           session_ptmap_id;
 
-  RTSPConnection  *connection;
+  GstRTSPConnection  *connection;
 
-  RTSPExtensionCtx *extension;
+  /* a list of RTSP extensions as GstElement */
+  GstRTSPExtensionList  *extensions;
 };
 
 struct _GstRTSPSrcClass {
@@ -177,10 +198,6 @@ struct _GstRTSPSrcClass {
 
 GType gst_rtspsrc_get_type(void);
 
-gboolean   gst_rtspsrc_send 		(GstRTSPSrc * src, RTSPMessage * request,
-		   			 RTSPMessage * response, RTSPStatusCode * code);
-
-
 G_END_DECLS
 
 #endif /* __GST_RTSPSRC_H__ */
