@@ -1,29 +1,26 @@
 /* GStreamer
-* Copyright (C) 2005 Sebastien Moutte <sebastien@moutte.net>
-* Copyright (C) 2007 Pioneers of the Inevitable <songbird@songbirdnest.com>
-*	
-* Based on directfb video sink
-* gstdirectdrawsink.c:
-*
-* This library is free software; you can redistribute it and/or
-* modify it under the terms of the GNU Library General Public
-* License as published by the Free Software Foundation; either
-* version 2 of the License, or (at your option) any later version.
-*
-* This library is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-* Library General Public License for more details.
-*
-* You should have received a copy of the GNU Library General Public
-* License along with this library; if not, write to the
-* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-* Boston, MA 02111-1307, USA.
-*
-* The development of this code was made possible due to the involvement
-* of Pioneers of the Inevitable, the creators of the Songbird Music player
-*
-*/
+ * Copyright (C) 2005 Sebastien Moutte <sebastien@moutte.net>
+ * Copyright (C) 2007 Pioneers of the Inevitable <songbird@songbirdnest.com>
+ *	
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * The development of this code was made possible due to the involvement
+ * of Pioneers of the Inevitable, the creators of the Songbird Music player
+ *
+ */
 
 /**
  * SECTION:element-directdrawsink
@@ -149,13 +146,24 @@ gst_directdraw_sink_set_window_id (GstXOverlay * overlay, ULONG window_id)
 {
   GstDirectDrawSink *ddrawsink = GST_DIRECTDRAW_SINK (overlay);
 
+  GST_OBJECT_LOCK (ddrawsink);
   /* check if we are already using this window id */
-  if (ddrawsink->video_window == (HWND) window_id)
+  if (ddrawsink->video_window == (HWND) window_id) {
+    GST_OBJECT_UNLOCK (ddrawsink);
     return;
+  }
 
   if (window_id) {
     HRESULT hres;
 
+    /* If we had an internal window, close it first */
+    if (ddrawsink->video_window && ddrawsink->our_video_window) {
+      /* Trick to let the event thread know that it has to die silently */
+      ddrawsink->our_video_window = FALSE;
+      /* Post quit message and wait for our event window thread */
+      PostMessage (ddrawsink->video_window, WM_QUIT, 0, 0);
+    }
+
     ddrawsink->video_window = (HWND) window_id;
     ddrawsink->our_video_window = FALSE;
     if (ddrawsink->setup) {
@@ -164,6 +172,9 @@ gst_directdraw_sink_set_window_id (GstXOverlay * overlay, ULONG window_id)
           ddrawsink->video_window);
     }
   }
+  /* FIXME: Handle the case where window_id is 0 and we want the sink to 
+   * create a new window when playback was already started (after set_caps) */
+  GST_OBJECT_UNLOCK (ddrawsink);
 }
 
 static void
@@ -203,6 +214,7 @@ gst_directdraw_sink_init_interfaces (GType type)
 
 /* Subclass of GstBuffer which manages buffer_pool surfaces lifetime    */
 static void gst_ddrawsurface_finalize (GstMiniObject * mini_object);
+static GstBufferClass *ddrawsurface_parent_class = NULL;
 
 static void
 gst_ddrawsurface_init (GstDDrawSurface * surface, gpointer g_class)
@@ -221,6 +233,8 @@ gst_ddrawsurface_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  ddrawsurface_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = GST_DEBUG_FUNCPTR (gst_ddrawsurface_finalize);
 }
 
@@ -272,7 +286,7 @@ gst_ddrawsurface_finalize (GstMiniObject * mini_object)
         surface->width, surface->height, ddrawsink->video_width,
         ddrawsink->video_height);
     gst_directdraw_sink_surface_destroy (ddrawsink, surface);
-
+    GST_MINI_OBJECT_CLASS (ddrawsurface_parent_class)->finalize (mini_object);
   } else {
     /* In that case we can reuse the image and add it to our image pool. */
     GST_CAT_INFO_OBJECT (directdrawsink_debug, ddrawsink,
@@ -285,10 +299,12 @@ gst_ddrawsurface_finalize (GstMiniObject * mini_object)
     ddrawsink->buffer_pool = g_slist_prepend (ddrawsink->buffer_pool, surface);
     g_mutex_unlock (ddrawsink->pool_lock);
   }
+
   return;
 
 no_sink:
   GST_CAT_WARNING (directdrawsink_debug, "no sink found");
+  GST_MINI_OBJECT_CLASS (ddrawsurface_parent_class)->finalize (mini_object);
   return;
 }
 
@@ -396,6 +412,8 @@ gst_directdraw_sink_finalize (GObject * object)
   if (ddrawsink->setup) {
     gst_directdraw_sink_cleanup (ddrawsink);
   }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -479,6 +497,11 @@ gst_directdraw_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
     gst_x_overlay_prepare_xwindow_id (GST_X_OVERLAY (ddrawsink));
   }
 
+  /* If we still don't have a window at that stage we create our own */
+  if (!ddrawsink->video_window) {
+    gst_directdraw_sink_create_default_window (ddrawsink);
+  }
+
   /* if we are rendering to our own window, resize it to video size */
   if (ddrawsink->video_window && ddrawsink->our_video_window) {
     SetWindowPos (ddrawsink->video_window, NULL,
@@ -487,6 +510,12 @@ gst_directdraw_sink_set_caps (GstBaseSink * bsink, GstCaps * caps)
         (GetSystemMetrics (SM_CYSIZEFRAME) * 2), SWP_SHOWWINDOW | SWP_NOMOVE);
   }
 
+  /* release the surface, we have to recreate it! */
+  if (ddrawsink->offscreen_surface) {
+    IDirectDrawSurface7_Release (ddrawsink->offscreen_surface);
+    ddrawsink->offscreen_surface = NULL;
+  }
+
   /* create an offscreen surface with the caps */
   ret = gst_directdraw_sink_check_offscreen_surface (ddrawsink);
   if (!ret) {
@@ -502,26 +531,28 @@ static GstStateChangeReturn
 gst_directdraw_sink_change_state (GstElement * element,
     GstStateChange transition)
 {
-  GstDirectDrawSink *ddrawsink = GST_DIRECTDRAW_SINK (element);;
+  GstDirectDrawSink *ddrawsink = GST_DIRECTDRAW_SINK (element);
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      if (ddrawsink->video_window == NULL)
-        if (!gst_directdraw_sink_create_default_window (ddrawsink))
-          return GST_STATE_CHANGE_FAILURE;
+      if (!gst_directdraw_sink_setup_ddraw (ddrawsink)) {
+        ret = GST_STATE_CHANGE_FAILURE;
+        goto beach;
+      }
 
-      if (!gst_directdraw_sink_setup_ddraw (ddrawsink))
-        return GST_STATE_CHANGE_FAILURE;
+      if (!(ddrawsink->caps = gst_directdraw_sink_get_ddrawcaps (ddrawsink))) {
+        ret = GST_STATE_CHANGE_FAILURE;
+        goto beach;
+      }
+      break;
+    default:
+      break;
+  }
 
-      if (!(ddrawsink->caps = gst_directdraw_sink_get_ddrawcaps (ddrawsink)))
-        return GST_STATE_CHANGE_FAILURE;
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      break;
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       ddrawsink->fps_n = 0;
       ddrawsink->fps_d = 1;
@@ -534,9 +565,12 @@ gst_directdraw_sink_change_state (GstElement * element,
       if (ddrawsink->setup)
         gst_directdraw_sink_cleanup (ddrawsink);
       break;
+    default:
+      break;
   }
 
-  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+beach:
+  return ret;
 }
 
 static GstFlowReturn
@@ -544,13 +578,23 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
     guint size, GstCaps * caps, GstBuffer ** buf)
 {
   GstDirectDrawSink *ddrawsink = GST_DIRECTDRAW_SINK (bsink);
+  GstStructure *structure;
+  gint width, height;
   GstDDrawSurface *surface = NULL;
   GstFlowReturn ret = GST_FLOW_OK;
   GstCaps *buffer_caps = caps;
   gboolean buffercaps_unref = FALSE;
 
   GST_CAT_INFO_OBJECT (directdrawsink_debug, ddrawsink,
-      "a buffer of %d bytes was requested", size);
+      "a buffer of %u bytes was requested", size);
+
+  structure = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (structure, "width", &width) ||
+      !gst_structure_get_int (structure, "height", &height)) {
+    GST_WARNING_OBJECT (ddrawsink, "invalid caps for buffer allocation %"
+        GST_PTR_FORMAT, caps);
+    return GST_FLOW_UNEXPECTED;
+  }
 
   g_mutex_lock (ddrawsink->pool_lock);
 
@@ -563,8 +607,8 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
           ddrawsink->buffer_pool);
 
       /* If the surface is invalid for our need, destroy */
-      if ((surface->width != ddrawsink->video_width) ||
-          (surface->height != ddrawsink->video_height) ||
+      if ((surface->width != width) ||
+          (surface->height != height) ||
           (memcmp (&surface->dd_pixel_format, &ddrawsink->dd_pixel_format,
                   sizeof (DDPIXELFORMAT)) ||
               !gst_directdraw_sink_surface_check (ddrawsink, surface))
@@ -587,9 +631,7 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
     HRESULT hres;
     DDSURFACEDESC2 surface_desc;
     DDSURFACEDESC2 *sd;
-    GstStructure *structure = NULL;
 
-    structure = gst_caps_get_structure (caps, 0);
     if (!gst_structure_get_int (structure, "depth", &depth)) {
       GST_CAT_DEBUG_OBJECT (directdrawsink_debug, ddrawsink,
           "Can't get depth from buffer_alloc caps");
@@ -645,7 +687,7 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
             buffer_caps = copy_caps;
             buffercaps_unref = TRUE;
             /* update buffer size needed to store video frames according to new caps */
-            size = ddrawsink->video_width * ddrawsink->video_height * (bpp / 8);
+            size = width * height * (bpp / 8);
 
             /* update our member pixel format */
             gst_ddrawvideosink_get_format_from_caps (ddrawsink, buffer_caps,
@@ -677,6 +719,7 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
 
   /* Now we should have a surface, set appropriate caps on it */
   if (surface) {
+    GST_BUFFER_FLAGS (GST_BUFFER (surface)) = 0;
     gst_buffer_set_caps (GST_BUFFER (surface), buffer_caps);
   }
 
@@ -690,6 +733,63 @@ gst_directdraw_sink_buffer_alloc (GstBaseSink * bsink, guint64 offset,
   return ret;
 }
 
+static void
+gst_directdraw_sink_draw_borders (GstDirectDrawSink * ddrawsink, RECT dst_rect)
+{
+  RECT win_rect, fill_rect;
+  POINT win_point;
+  HDC hdc;
+
+  g_return_if_fail (GST_IS_DIRECTDRAW_SINK (ddrawsink));
+
+  /* Get the target window rect */
+  win_point.x = 0;
+  win_point.y = 0;
+  ClientToScreen (ddrawsink->video_window, &win_point);
+  GetClientRect (ddrawsink->video_window, &win_rect);
+  OffsetRect (&win_rect, win_point.x, win_point.y);
+
+  /* We acquire a drawing context */
+  if (IDirectDrawSurface7_GetDC (ddrawsink->primary_surface, &hdc) == DD_OK) {
+    HBRUSH brush = CreateSolidBrush (RGB (0, 0, 0));
+
+    /* Left border */
+    if (dst_rect.left > win_rect.left) {
+      fill_rect.left = win_rect.left;
+      fill_rect.top = win_rect.top;
+      fill_rect.bottom = win_rect.bottom;
+      fill_rect.right = dst_rect.left;
+      FillRect (hdc, &fill_rect, brush);
+    }
+    /* Right border */
+    if (dst_rect.right < win_rect.right) {
+      fill_rect.top = win_rect.top;
+      fill_rect.left = dst_rect.right;
+      fill_rect.bottom = win_rect.bottom;
+      fill_rect.right = win_rect.right;
+      FillRect (hdc, &fill_rect, brush);
+    }
+    /* Top border */
+    if (dst_rect.top > win_rect.top) {
+      fill_rect.top = win_rect.top;
+      fill_rect.left = win_rect.left;
+      fill_rect.right = win_rect.right;
+      fill_rect.bottom = dst_rect.top;
+      FillRect (hdc, &fill_rect, brush);
+    }
+    /* Bottom border */
+    if (dst_rect.bottom < win_rect.bottom) {
+      fill_rect.top = dst_rect.bottom;
+      fill_rect.left = win_rect.left;
+      fill_rect.right = win_rect.right;
+      fill_rect.bottom = win_rect.bottom;
+      FillRect (hdc, &fill_rect, brush);
+    }
+    DeleteObject (brush);
+    IDirectDrawSurface7_ReleaseDC (ddrawsink->primary_surface, hdc);
+  }
+}
+
 static GstFlowReturn
 gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 {
@@ -710,10 +810,25 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     /* use last buffer */
     buf = ddrawsink->last_buffer;
   }
-  if (buf == NULL)
+
+  if (buf == NULL) {
+    GST_ERROR_OBJECT (ddrawsink, "No buffer to render.");
     return GST_FLOW_ERROR;
+  } else if (!ddrawsink->video_window) {
+    GST_WARNING_OBJECT (ddrawsink, "No video window to render to.");
+    return GST_FLOW_ERROR;
+  }
 
   /* get the video window position */
+  GST_OBJECT_LOCK (ddrawsink);
+  if (G_UNLIKELY (!ddrawsink->video_window)) {
+    GST_OBJECT_UNLOCK (ddrawsink);
+    GST_CAT_WARNING_OBJECT (directdrawsink_debug, ddrawsink,
+        "gst_directdraw_sink_show_frame our video window disappeared");
+    GST_ELEMENT_ERROR (ddrawsink, RESOURCE, NOT_FOUND,
+        ("Output window was closed"), (NULL));
+    return GST_FLOW_ERROR;
+  }
   dest_surf_point.x = 0;
   dest_surf_point.y = 0;
   ClientToScreen (ddrawsink->video_window, &dest_surf_point);
@@ -728,7 +843,9 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     src_rect.right = ddrawsink->video_width;
     gst_directdraw_sink_center_rect (ddrawsink, src_rect, destsurf_rect,
         &destsurf_rect);
+    gst_directdraw_sink_draw_borders (ddrawsink, destsurf_rect);
   }
+  GST_OBJECT_UNLOCK (ddrawsink);
 
   if (ddrawsink->must_recreate_offscreen && ddrawsink->offscreen_surface) {
     IDirectDrawSurface7_Release (ddrawsink->offscreen_surface);
@@ -763,7 +880,11 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       GST_CAT_WARNING_OBJECT (directdrawsink_debug, ddrawsink,
           "gst_directdraw_sink_show_frame failed locking surface %s",
           DDErrorString (hRes));
-      return GST_FLOW_ERROR;
+
+      if (IDirectDrawSurface7_IsLost (ddrawsink->offscreen_surface) == DD_OK)
+        return GST_FLOW_OK;
+      else
+        return GST_FLOW_ERROR;
     }
 
     /* Write each line respecting the destination surface pitch */
@@ -783,17 +904,18 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       return GST_FLOW_ERROR;
     }
 
-    /* blit to primary surface ( Blt will scale the video the dest rect surface if needed */
+    /* blit to primary surface ( Blt will scale the video the dest rect surface
+     * if needed */
     hRes = IDirectDrawSurface7_Blt (ddrawsink->primary_surface, &destsurf_rect,
         ddrawsink->offscreen_surface, NULL, DDBLT_WAIT, NULL);
-    if (hRes != DD_OK)
+    if (hRes != DD_OK)          /* FIXME: Is it really safe to continue past here ? */
       GST_CAT_WARNING_OBJECT (directdrawsink_debug, ddrawsink,
           "IDirectDrawSurface7_Blt (object's offscreen surface) " "returned %s",
           DDErrorString (hRes));
 
   } else {
-    /* We are receiving a directdraw surface (previously returned by our buffer pool
-       So we will simply blit it on the primary surface */
+    /* We are receiving a directdraw surface (previously returned by our buffer
+     * pool so we will simply blit it on the primary surface */
     GstDDrawSurface *surface = NULL;
 
     surface = GST_DDRAWSURFACE (buf);
@@ -805,7 +927,7 @@ gst_directdraw_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
     /* blit to our primary surface */
     hRes = IDirectDrawSurface7_Blt (ddrawsink->primary_surface, &destsurf_rect,
         surface->surface, NULL, DDBLT_WAIT, NULL);
-    if (hRes != DD_OK)
+    if (hRes != DD_OK)          /* FIXME: Is it really safe to continue past here ? */
       GST_CAT_WARNING_OBJECT (directdrawsink_debug, ddrawsink,
           "IDirectDrawSurface7_Blt (offscreen surface from buffer_alloc) "
           "returned %s", DDErrorString (hRes));
@@ -1161,8 +1283,8 @@ gst_directdraw_sink_setup_ddraw (GstDirectDrawSink * ddrawsink)
   gboolean bRet = TRUE;
   HRESULT hRes;
 
-  /* create an instance of the ddraw object use DDCREATE_EMULATIONONLY as first parameter to
-     force Directdraw to use the hardware emulation layer */
+  /* create an instance of the ddraw object use DDCREATE_EMULATIONONLY as first
+   * parameter to force Directdraw to use the hardware emulation layer */
   hRes = DirectDrawCreateEx ( /*DDCREATE_EMULATIONONLY */ 0,
       (void **) &ddrawsink->ddraw_object, &IID_IDirectDraw7, NULL);
   if (hRes != DD_OK || ddrawsink->ddraw_object == NULL) {
@@ -1174,7 +1296,7 @@ gst_directdraw_sink_setup_ddraw (GstDirectDrawSink * ddrawsink)
 
   /* set cooperative level */
   hRes = IDirectDraw7_SetCooperativeLevel (ddrawsink->ddraw_object,
-      ddrawsink->video_window, DDSCL_NORMAL);
+      NULL, DDSCL_NORMAL);
   if (hRes != DD_OK) {
     GST_ELEMENT_ERROR (ddrawsink, RESOURCE, WRITE,
         ("Failed to set the set the cooperative level error=%s",
@@ -1185,10 +1307,9 @@ gst_directdraw_sink_setup_ddraw (GstDirectDrawSink * ddrawsink)
   /* setup the clipper object */
   hRes = IDirectDraw7_CreateClipper (ddrawsink->ddraw_object, 0,
       &ddrawsink->clipper, NULL);
-  if (hRes == DD_OK) {
-    hRes = IDirectDrawClipper_SetHWnd (ddrawsink->clipper, 0,
-        ddrawsink->video_window);
-  }
+
+  if (hRes == DD_OK && ddrawsink->video_window)
+    IDirectDrawClipper_SetHWnd (ddrawsink->clipper, 0, ddrawsink->video_window);
 
   /* create our primary surface */
   if (!gst_directdraw_sink_check_primary_surface (ddrawsink))
@@ -1206,13 +1327,13 @@ WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
   switch (message) {
     case WM_ERASEBKGND:
       return TRUE;
-    case WM_DESTROY:
-      PostQuitMessage (0);
-      break;
     case WM_CLOSE:
       DestroyWindow (hWnd);
+    case WM_DESTROY:
+      PostQuitMessage (0);
       return 0;
   }
+
   return DefWindowProc (hWnd, message, wParam, lParam);
 }
 
@@ -1220,6 +1341,7 @@ static gpointer
 gst_directdraw_sink_window_thread (GstDirectDrawSink * ddrawsink)
 {
   WNDCLASS WndClass;
+  MSG msg;
 
   memset (&WndClass, 0, sizeof (WNDCLASS));
   WndClass.style = CS_HREDRAW | CS_VREDRAW;
@@ -1237,23 +1359,33 @@ gst_directdraw_sink_window_thread (GstDirectDrawSink * ddrawsink)
       WS_OVERLAPPEDWINDOW | WS_SIZEBOX, 0, 0, 640, 480, NULL, NULL,
       WndClass.hInstance, NULL);
   if (ddrawsink->video_window == NULL)
-    return FALSE;
+    return NULL;
 
-  /* signal application we create a window */
+  /* Set the clipper on that window */
+  IDirectDrawClipper_SetHWnd (ddrawsink->clipper, 0, ddrawsink->video_window);
+
+  /* signal application we created a window */
   gst_x_overlay_got_xwindow_id (GST_X_OVERLAY (ddrawsink),
       (gulong) ddrawsink->video_window);
 
   ReleaseSemaphore (ddrawsink->window_created_signal, 1, NULL);
 
   /* start message loop processing our default window messages */
-  while (1) {
-    MSG msg;
-
-    if (!GetMessage (&msg, ddrawsink->video_window, 0, 0))
-      break;
+  while (GetMessage (&msg, NULL, 0, 0) != FALSE) {
+    TranslateMessage (&msg);
     DispatchMessage (&msg);
   }
 
+  GST_CAT_LOG_OBJECT (directdrawsink_debug, ddrawsink,
+      "our window received WM_QUIT or error.");
+  /* The window could have changed, if it is not ours anymore we don't
+   * overwrite the current video window with NULL */
+  if (ddrawsink->our_video_window) {
+    GST_OBJECT_LOCK (ddrawsink);
+    ddrawsink->video_window = NULL;
+    GST_OBJECT_UNLOCK (ddrawsink);
+  }
+
   return NULL;
 }
 
@@ -1514,7 +1646,8 @@ gst_directdraw_sink_get_ddrawcaps (GstDirectDrawSink * ddrawsink)
   IDirectDraw7_GetCaps (ddrawsink->ddraw_object, &ddcaps_hardware,
       &ddcaps_emulation);
 
-  /*  we don't test for DDCAPS_BLTSTRETCH on the hardware as the directdraw emulation layer can do it */
+  /* we don't test for DDCAPS_BLTSTRETCH on the hardware as the directdraw 
+   * emulation layer can do it */
   if (!(ddcaps_hardware.dwCaps & DDCAPS_BLTFOURCC)) {
     DDSURFACEDESC2 surface_desc;
     DDSURFACEDESC2 *sd;
@@ -1577,7 +1710,8 @@ gst_directdraw_sink_get_ddrawcaps (GstDirectDrawSink * ddrawsink)
     return NULL;
   }
 
-  /*GST_CAT_INFO_OBJECT (directdrawsink_debug, ddrawsink, "returning caps %s", gst_caps_to_string (ddrawsink->caps)); */
+  /*GST_CAT_INFO_OBJECT (directdrawsink_debug, ddrawsink, "returning caps %s",
+   * gst_caps_to_string (ddrawsink->caps)); */
 
   return ddrawsink->caps;
 }
@@ -1622,12 +1756,13 @@ gst_directdraw_sink_surface_create (GstDirectDrawSink * ddrawsink,
         "failed getting pixel format from caps %" GST_PTR_FORMAT, caps);
   }
 
-  /* disable return of directdraw surface to buffer alloc because actually I have no solution
-   * to handle display mode changes. The problem is that when the display mode is changed
-   * surface's memory is freed then the upstream filter would crash trying to write to this memory.
-   * Directdraw has a system lock (DDLOCK_NOSYSLOCK to disable it) to prevent display mode changes 
-   * when a surface memory is locked but we need to disable this lock to return multiple buffers (surfaces)
-   * and do not lock directdraw API calls.
+  /* disable return of directdraw surface to buffer alloc because actually I
+   * have no solution to handle display mode changes. The problem is that when
+   * the display mode is changed surface's memory is freed then the upstream
+   * filter would crash trying to write to this memory. Directdraw has a system
+   * lock (DDLOCK_NOSYSLOCK to disable it) to prevent display mode changes 
+   * when a surface memory is locked but we need to disable this lock to return
+   * multiple buffers (surfaces) and do not lock directdraw API calls.
    */
 #if 0
 /*  if (ddrawsink->ddraw_object) {*/
