@@ -67,6 +67,8 @@
 #include "gstosssrc.h"
 #include "common.h"
 
+#include <gst/gst-i18n-plugin.h>
+
 GST_DEBUG_CATEGORY_EXTERN (oss_debug);
 #define GST_CAT_DEFAULT oss_debug
 
@@ -230,11 +232,18 @@ gst_oss_src_get_property (GObject * object, guint prop_id,
 static void
 gst_oss_src_init (GstOssSrc * osssrc, GstOssSrcClass * g_class)
 {
+  const gchar *device;
+
   GST_DEBUG ("initializing osssrc");
 
+  device = g_getenv ("AUDIODEV");
+  if (device == NULL)
+    device = DEFAULT_DEVICE;
+
   osssrc->fd = -1;
-  osssrc->device = g_strdup (DEFAULT_DEVICE);
+  osssrc->device = g_strdup (device);
   osssrc->device_name = g_strdup (DEFAULT_DEVICE_NAME);
+  osssrc->probed_caps = NULL;
 }
 
 static void
@@ -255,12 +264,23 @@ gst_oss_src_getcaps (GstBaseSrc * bsrc)
   osssrc = GST_OSS_SRC (bsrc);
 
   if (osssrc->fd == -1) {
-    caps = gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC_PAD
-            (bsrc)));
-  } else {
-    caps = gst_oss_helper_probe_caps (osssrc->fd);
+    GST_DEBUG_OBJECT (osssrc, "device not open, using template caps");
+    return NULL;                /* base class will get template caps for us */
   }
 
+  if (osssrc->probed_caps) {
+    GST_LOG_OBJECT (osssrc, "Returning cached caps");
+    return gst_caps_ref (osssrc->probed_caps);
+  }
+
+  caps = gst_oss_helper_probe_caps (osssrc->fd);
+
+  if (caps) {
+    osssrc->probed_caps = gst_caps_ref (caps);
+  }
+
+  GST_INFO_OBJECT (osssrc, "returning caps %" GST_PTR_FORMAT, caps);
+
   return caps;
 }
 
@@ -336,8 +356,14 @@ gst_oss_src_open (GstAudioSrc * asrc)
   mode |= O_NONBLOCK;
 
   oss->fd = open (oss->device, mode, 0);
-  if (oss->fd == -1)
-    goto open_failed;
+  if (oss->fd == -1) {
+    switch (errno) {
+      case EACCES:
+        goto no_permission;
+      default:
+        goto open_failed;
+    }
+  }
 
   if (!oss->mixer) {
     oss->mixer = gst_ossmixer_new ("/dev/mixer", GST_OSS_MIXER_CAPTURE);
@@ -349,11 +375,20 @@ gst_oss_src_open (GstAudioSrc * asrc)
   }
   return TRUE;
 
+no_permission:
+  {
+    GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
+        (_("Could not open audio device for recording."
+                "You don't have permission to open the device.")),
+        GST_ERROR_SYSTEM);
+    return FALSE;
+  }
 open_failed:
   {
     GST_ELEMENT_ERROR (oss, RESOURCE, OPEN_READ,
+        (_("Could not open audio device for recording.")),
         ("Unable to open device %s for recording: %s",
-            oss->device, g_strerror (errno)), (NULL));
+            oss->device, g_strerror (errno)));
     return FALSE;
   }
 }
@@ -372,6 +407,8 @@ gst_oss_src_close (GstAudioSrc * asrc)
     oss->mixer = NULL;
   }
 
+  gst_caps_replace (&oss->probed_caps, NULL);
+
   return TRUE;
 }
 
@@ -505,12 +542,7 @@ gst_oss_src_delay (GstAudioSrc * asrc)
 static void
 gst_oss_src_reset (GstAudioSrc * asrc)
 {
-  GstOssSrc *oss;
-
-  //gint ret;
-
-  oss = GST_OSS_SRC (asrc);
-
-  /* deadlocks on my machine... */
-  //ret = ioctl (oss->fd, SNDCTL_DSP_RESET, 0);
+  /* There's nothing we can do here really: OSS can't handle access to the
+   * same device/fd from multiple threads and might deadlock or blow up in
+   * other ways if we try an ioctl SNDCTL_DSP_RESET or similar */
 }
