@@ -37,10 +37,14 @@
  * errors here, check your linux/time.h && sys/time.h header setup.
  */
 #include <sys/types.h>
+#ifndef __sun
 #include <linux/types.h>
 #define _LINUX_TIME_H
 #define __user
 #include <linux/videodev2.h>
+#else
+#include <sys/videodev2.h>
+#endif
 
 #include <gst/gst.h>
 #include <gst/base/gstpushsrc.h>
@@ -110,6 +114,7 @@ GType gst_v4l2_object_get_type (void);
 #define V4L2_STD_OBJECT_PROPS		\
     PROP_DEVICE,			\
     PROP_DEVICE_NAME,			\
+    PROP_DEVICE_FD,			\
     PROP_FLAGS
 
 /* create/destroy */
