@@ -36,6 +36,11 @@
 #include "v4l2src_calls.h"
 #include <sys/time.h>
 #include <unistd.h>
+#ifdef __sun
+/* Needed on older Solaris Nevada builds (72 at least) */
+#include <stropts.h>
+#include <sys/ioccom.h>
+#endif
 
 #include "gstv4l2tuner.h"
 
@@ -46,11 +51,6 @@ GST_DEBUG_CATEGORY_EXTERN (v4l2src_debug);
 #define GST_V4L2_SET_ACTIVE(element) (element)->buffer = GINT_TO_POINTER (-1)
 #define GST_V4L2_SET_INACTIVE(element) (element)->buffer = NULL
 
-#define DEBUG(format, args...) \
-	GST_CAT_DEBUG_OBJECT (\
-		v4l2src_debug, v4l2src, \
-		"V4L2SRC: " format, ##args)
-
 /* On some systems MAP_FAILED seems to be missing */
 #ifndef MAP_FAILED
 #define MAP_FAILED ((caddr_t) -1)
@@ -61,15 +61,25 @@ GST_DEBUG_CATEGORY_EXTERN (v4l2src_debug);
 #define GST_IS_V4L2_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_V4L2_BUFFER))
 #define GST_V4L2_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_V4L2_BUFFER, GstV4l2Buffer))
 
+static GstBufferClass *v4l2buffer_parent_class = NULL;
+
+/* Local functions */
+static gboolean
+gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
+    gint * width, gint * height);
+
 static void
 gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
 {
   GstV4l2BufferPool *pool;
   gboolean resuscitated = FALSE;
+  gint index;
 
   pool = buffer->pool;
 
-  GST_LOG ("finalizing buffer %d", buffer->vbuffer.index);
+  index = buffer->vbuffer.index;
+
+  GST_LOG ("finalizing buffer %p %d", buffer, index);
 
   g_mutex_lock (pool->lock);
   if (GST_BUFFER_SIZE (buffer) != 0)
@@ -78,20 +88,27 @@ gst_v4l2_buffer_finalize (GstV4l2Buffer * buffer)
 
   if (pool->running) {
     if (ioctl (pool->video_fd, VIDIOC_QBUF, &buffer->vbuffer) < 0) {
-      GST_WARNING ("could not requeue buffer %d", buffer->vbuffer.index);
+      GST_WARNING ("could not requeue buffer %p %d", buffer, index);
     } else {
       /* FIXME: check that the caps didn't change */
+      GST_LOG ("reviving buffer %p, %d", buffer, index);
       gst_buffer_ref (GST_BUFFER (buffer));
       GST_BUFFER_SIZE (buffer) = 0;
-      pool->buffers[buffer->vbuffer.index] = buffer;
+      pool->buffers[index] = buffer;
       resuscitated = TRUE;
     }
+  } else {
+    GST_LOG ("the pool is shutting down");
   }
   g_mutex_unlock (pool->lock);
 
   if (!resuscitated) {
+    GST_LOG ("buffer %p not recovered, unmapping", buffer);
     gst_mini_object_unref (GST_MINI_OBJECT (pool));
-    munmap (GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
+    munmap ((void *) GST_BUFFER_DATA (buffer), buffer->vbuffer.length);
+
+    GST_MINI_OBJECT_CLASS (v4l2buffer_parent_class)->finalize (GST_MINI_OBJECT
+        (buffer));
   }
 }
 
@@ -106,6 +123,8 @@ gst_v4l2_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  v4l2buffer_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_v4l2_buffer_finalize;
 }
@@ -138,12 +157,16 @@ static GstV4l2Buffer *
 gst_v4l2_buffer_new (GstV4l2BufferPool * pool, guint index, GstCaps * caps)
 {
   GstV4l2Buffer *ret;
+  guint8 *data;
 
   ret = (GstV4l2Buffer *) gst_mini_object_new (GST_TYPE_V4L2_BUFFER);
 
+  GST_LOG ("creating buffer %u, %p in pool %p", index, ret, pool);
+
   ret->pool = pool;
   gst_mini_object_ref (GST_MINI_OBJECT (pool));
   memset (&ret->vbuffer, 0x00, sizeof (ret->vbuffer));
+
   ret->vbuffer.index = index;
   ret->vbuffer.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
   ret->vbuffer.memory = V4L2_MEMORY_MMAP;
@@ -151,18 +174,34 @@ gst_v4l2_buffer_new (GstV4l2BufferPool * pool, guint index, GstCaps * caps)
   if (ioctl (pool->video_fd, VIDIOC_QUERYBUF, &ret->vbuffer) < 0)
     goto querybuf_failed;
 
-  GST_BUFFER_DATA (ret) = mmap (0, ret->vbuffer.length,
+  GST_LOG ("  index:     %u", ret->vbuffer.index);
+  GST_LOG ("  type:      %d", ret->vbuffer.type);
+  GST_LOG ("  bytesused: %u", ret->vbuffer.bytesused);
+  GST_LOG ("  flags:     %08x", ret->vbuffer.flags);
+  GST_LOG ("  field:     %d", ret->vbuffer.field);
+  GST_LOG ("  memory:    %d", ret->vbuffer.memory);
+  if (ret->vbuffer.memory == V4L2_MEMORY_MMAP)
+    GST_LOG ("  MMAP offset:  %u", ret->vbuffer.m.offset);
+  GST_LOG ("  length:    %u", ret->vbuffer.length);
+  GST_LOG ("  input:     %u", ret->vbuffer.input);
+
+  data = (guint8 *) mmap (0, ret->vbuffer.length,
       PROT_READ | PROT_WRITE, MAP_SHARED, pool->video_fd,
       ret->vbuffer.m.offset);
 
-  if (GST_BUFFER_DATA (ret) == MAP_FAILED)
+  if (data == MAP_FAILED)
     goto mmap_failed;
 
+  GST_BUFFER_DATA (ret) = data;
+  GST_BUFFER_SIZE (ret) = ret->vbuffer.length;
+
   GST_BUFFER_FLAG_SET (ret, GST_BUFFER_FLAG_READONLY);
+
   gst_buffer_set_caps (GST_BUFFER (ret), caps);
 
   return ret;
 
+  /* ERRORS */
 querybuf_failed:
   {
     gint errnosave = errno;
@@ -187,6 +226,8 @@ mmap_failed:
 #define GST_IS_V4L2_BUFFER_POOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_V4L2_BUFFER_POOL))
 #define GST_V4L2_BUFFER_POOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_V4L2_BUFFER_POOL, GstV4l2BufferPool))
 
+static GstMiniObjectClass *buffer_pool_parent_class = NULL;
+
 static void
 gst_v4l2_buffer_pool_finalize (GstV4l2BufferPool * pool)
 {
@@ -199,6 +240,8 @@ gst_v4l2_buffer_pool_finalize (GstV4l2BufferPool * pool)
   if (pool->buffers)
     g_free (pool->buffers);
   pool->buffers = NULL;
+  GST_MINI_OBJECT_CLASS (buffer_pool_parent_class)->finalize (GST_MINI_OBJECT
+      (pool));
 }
 
 static void
@@ -214,6 +257,8 @@ gst_v4l2_buffer_pool_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  buffer_pool_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize = (GstMiniObjectFinalizeFunction)
       gst_v4l2_buffer_pool_finalize;
 }
@@ -266,6 +311,7 @@ gst_v4l2_buffer_pool_new (GstV4l2Src * v4l2src, gint fd, gint num_buffers,
 
   return pool;
 
+  /* ERRORS */
 dup_failed:
   {
     gint errnosave = errno;
@@ -296,18 +342,22 @@ gst_v4l2_buffer_pool_activate (GstV4l2BufferPool * pool, GstV4l2Src * v4l2src)
   g_mutex_lock (pool->lock);
 
   for (n = 0; n < pool->buffer_count; n++) {
-    struct v4l2_buffer *buf = &pool->buffers[n]->vbuffer;
+    struct v4l2_buffer *buf;
+
+    buf = &pool->buffers[n]->vbuffer;
+
+    GST_LOG ("enqueue pool buffer %d", n);
 
     if (ioctl (pool->video_fd, VIDIOC_QBUF, buf) < 0)
       goto queue_failed;
   }
-
   pool->running = TRUE;
 
   g_mutex_unlock (pool->lock);
 
   return TRUE;
 
+  /* ERRORS */
 queue_failed:
   {
     GST_ELEMENT_ERROR (v4l2src, RESOURCE, READ,
@@ -348,6 +398,97 @@ gst_v4l2_buffer_pool_destroy (GstV4l2BufferPool * pool)
   gst_mini_object_unref (GST_MINI_OBJECT (pool));
 }
 
+/* complete made up ranking, the values themselves are meaningless */
+#define YUV_BASE_RANK     1000
+#define JPEG_BASE_RANK     500
+#define DV_BASE_RANK       200
+#define RGB_BASE_RANK      100
+#define YUV_ODD_BASE_RANK   50
+#define RGB_ODD_BASE_RANK   25
+#define BAYER_BASE_RANK     15
+#define GREY_BASE_RANK       5
+
+static gint
+gst_v4l2src_format_get_rank (guint32 fourcc)
+{
+  switch (fourcc) {
+    case V4L2_PIX_FMT_MJPEG:
+      return JPEG_BASE_RANK;
+    case V4L2_PIX_FMT_JPEG:
+      return JPEG_BASE_RANK + 1;
+
+    case V4L2_PIX_FMT_RGB332:
+    case V4L2_PIX_FMT_RGB555:
+    case V4L2_PIX_FMT_RGB555X:
+    case V4L2_PIX_FMT_RGB565:
+    case V4L2_PIX_FMT_RGB565X:
+      return RGB_ODD_BASE_RANK;
+
+    case V4L2_PIX_FMT_RGB24:
+    case V4L2_PIX_FMT_BGR24:
+      return RGB_BASE_RANK - 1;
+
+    case V4L2_PIX_FMT_RGB32:
+    case V4L2_PIX_FMT_BGR32:
+      return RGB_BASE_RANK;
+
+    case V4L2_PIX_FMT_GREY:    /*  8  Greyscale     */
+      return GREY_BASE_RANK;
+
+    case V4L2_PIX_FMT_NV12:    /* 12  Y/CbCr 4:2:0  */
+    case V4L2_PIX_FMT_NV21:    /* 12  Y/CrCb 4:2:0  */
+    case V4L2_PIX_FMT_YYUV:    /* 16  YUV 4:2:2     */
+    case V4L2_PIX_FMT_HI240:   /*  8  8-bit color   */
+      return YUV_ODD_BASE_RANK;
+
+    case V4L2_PIX_FMT_YVU410:  /* YVU9,  9 bits per pixel */
+      return YUV_BASE_RANK + 3;
+    case V4L2_PIX_FMT_YUV410:  /* YUV9,  9 bits per pixel */
+      return YUV_BASE_RANK + 2;
+    case V4L2_PIX_FMT_YUV420:  /* I420, 12 bits per pixel */
+      return YUV_BASE_RANK + 7;
+    case V4L2_PIX_FMT_YUYV:    /* YUY2, 16 bits per pixel */
+      return YUV_BASE_RANK + 10;
+    case V4L2_PIX_FMT_YVU420:  /* YV12, 12 bits per pixel */
+      return YUV_BASE_RANK + 6;
+    case V4L2_PIX_FMT_UYVY:    /* UYVY, 16 bits per pixel */
+      return YUV_BASE_RANK + 9;
+    case V4L2_PIX_FMT_Y41P:    /* Y41P, 12 bits per pixel */
+      return YUV_BASE_RANK + 5;
+    case V4L2_PIX_FMT_YUV411P: /* Y41B, 12 bits per pixel */
+      return YUV_BASE_RANK + 4;
+    case V4L2_PIX_FMT_YUV422P: /* Y42B, 16 bits per pixel */
+      return YUV_BASE_RANK + 8;
+
+    case V4L2_PIX_FMT_DV:
+      return DV_BASE_RANK;
+
+    case V4L2_PIX_FMT_MPEG:    /* MPEG          */
+    case V4L2_PIX_FMT_WNVA:    /* Winnov hw compres */
+      return 0;
+
+    case V4L2_PIX_FMT_SBGGR8:
+      return BAYER_BASE_RANK;
+
+    default:
+      break;
+  }
+
+  return 0;
+}
+
+static gint
+gst_v4l2src_format_cmp_func (gconstpointer a, gconstpointer b)
+{
+  guint32 pf1 = ((struct v4l2_fmtdesc *) a)->pixelformat;
+  guint32 pf2 = ((struct v4l2_fmtdesc *) b)->pixelformat;
+
+  if (pf1 == pf2)
+    return 0;
+
+  return gst_v4l2src_format_get_rank (pf2) - gst_v4l2src_format_get_rank (pf1);
+}
+
 /******************************************************
  * gst_v4l2src_fill_format_list():
  *   create list of supported capture formats
@@ -363,22 +504,36 @@ gst_v4l2src_fill_format_list (GstV4l2Src * v4l2src)
 
   /* format enumeration */
   for (n = 0;; n++) {
-    format = g_new (struct v4l2_fmtdesc, 1);
+    format = g_new0 (struct v4l2_fmtdesc, 1);
 
     format->index = n;
     format->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+
     if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_ENUM_FMT, format) < 0) {
       if (errno == EINVAL) {
+        g_free (format);
         break;                  /* end of enumeration */
-      } else
+      } else {
         goto failed;
+      }
     }
-    GST_DEBUG_OBJECT (v4l2src, "got format %" GST_FOURCC_FORMAT,
+
+    GST_LOG_OBJECT (v4l2src, "index:       %u", format->index);
+    GST_LOG_OBJECT (v4l2src, "type:        %d", format->type);
+    GST_LOG_OBJECT (v4l2src, "flags:       %08x", format->flags);
+    GST_LOG_OBJECT (v4l2src, "description: '%s'", format->description);
+    GST_LOG_OBJECT (v4l2src, "pixelformat: %" GST_FOURCC_FORMAT,
         GST_FOURCC_ARGS (format->pixelformat));
 
-    v4l2src->formats = g_slist_prepend (v4l2src->formats, format);
+    /* sort formats according to our preference;  we do this, because caps
+     * are probed in the order the formats are in the list, and the order of
+     * formats in the final probed caps matters for things like fixation */
+    v4l2src->formats = g_slist_insert_sorted (v4l2src->formats, format,
+        (GCompareFunc) gst_v4l2src_format_cmp_func);
   }
-  v4l2src->formats = g_slist_reverse (v4l2src->formats);
+
+  GST_DEBUG_OBJECT (v4l2src, "got %d format(s)", n);
+
   return TRUE;
 
   /* ERRORS */
@@ -413,14 +568,11 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
     guint32 pixelformat,
     guint32 width, guint32 height, const GstStructure * template)
 {
-  GstCaps *ret;
   gint fd = v4l2src->v4l2object->video_fd;
   struct v4l2_frmivalenum ival;
   guint32 num, denom;
   GstStructure *s;
-  GValue rate = { 0, };
-
-  ret = gst_caps_new_empty ();
+  GValue rates = { 0, };
 
   memset (&ival, 0, sizeof (struct v4l2_frmivalenum));
   ival.index = 0;
@@ -428,18 +580,19 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
   ival.width = width;
   ival.height = height;
 
+  GST_LOG_OBJECT (v4l2src, "get frame interval for %ux%u, %" GST_FOURCC_FORMAT,
+      width, height, GST_FOURCC_ARGS (pixelformat));
+
   /* keep in mind that v4l2 gives us frame intervals (durations); we invert the
    * fraction to get framerate */
-
   if (ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) < 0)
     goto enum_frameintervals_failed;
 
   if (ival.type == V4L2_FRMIVAL_TYPE_DISCRETE) {
-    GValue frac = { 0, };
+    GValue rate = { 0, };
 
-    g_value_init (&rate, GST_TYPE_LIST);
-
-    g_value_init (&frac, GST_TYPE_FRACTION);
+    g_value_init (&rates, GST_TYPE_LIST);
+    g_value_init (&rate, GST_TYPE_FRACTION);
 
     do {
       num = ival.discrete.numerator;
@@ -451,66 +604,100 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
         denom >>= 1;
       }
 
-      gst_value_set_fraction (&frac, denom, num);
-      gst_value_list_append_value (&rate, &frac);
+      GST_LOG_OBJECT (v4l2src, "adding discrete framerate: %d/%d", denom, num);
+
+      /* swap to get the framerate */
+      gst_value_set_fraction (&rate, denom, num);
+      gst_value_list_append_value (&rates, &rate);
 
       ival.index++;
     } while (ioctl (fd, VIDIOC_ENUM_FRAMEINTERVALS, &ival) >= 0);
   } else if (ival.type == V4L2_FRMIVAL_TYPE_STEPWISE) {
-    GValue frac = { 0, };
+    GValue min = { 0, };
     GValue step = { 0, };
     GValue max = { 0, };
+    gboolean added = FALSE;
+    guint32 minnum, mindenom;
+    guint32 maxnum, maxdenom;
 
-    g_value_init (&rate, GST_TYPE_LIST);
+    g_value_init (&rates, GST_TYPE_LIST);
 
-    g_value_init (&frac, GST_TYPE_FRACTION);
+    g_value_init (&min, GST_TYPE_FRACTION);
     g_value_init (&step, GST_TYPE_FRACTION);
     g_value_init (&max, GST_TYPE_FRACTION);
 
-    num = ival.stepwise.min.numerator;
-    denom = ival.stepwise.min.denominator;
-    if (num > G_MAXINT || denom > G_MAXINT) {
-      num >>= 1;
-      denom >>= 1;
+    /* get the min */
+    minnum = ival.stepwise.min.numerator;
+    mindenom = ival.stepwise.min.denominator;
+    if (minnum > G_MAXINT || mindenom > G_MAXINT) {
+      minnum >>= 1;
+      mindenom >>= 1;
     }
-    gst_value_set_fraction (&frac, num, denom);
+    GST_LOG_OBJECT (v4l2src, "stepwise min frame interval: %d/%d", minnum,
+        mindenom);
+    gst_value_set_fraction (&min, minnum, mindenom);
 
+    /* get the max */
+    maxnum = ival.stepwise.max.numerator;
+    maxdenom = ival.stepwise.max.denominator;
+    if (maxnum > G_MAXINT || maxdenom > G_MAXINT) {
+      maxnum >>= 1;
+      maxdenom >>= 1;
+    }
+
+    GST_LOG_OBJECT (v4l2src, "stepwise max frame interval: %d/%d", maxnum,
+        maxdenom);
+    gst_value_set_fraction (&max, maxnum, maxdenom);
+
+    /* get the step */
     num = ival.stepwise.step.numerator;
     denom = ival.stepwise.step.denominator;
     if (num > G_MAXINT || denom > G_MAXINT) {
       num >>= 1;
       denom >>= 1;
     }
+
+    if (num == 0 || denom == 0) {
+      /* in this case we have a wrong fraction or no step, set the step to max
+       * so that we only add the min value in the loop below */
+      num = maxnum;
+      denom = maxdenom;
+    }
+
     /* since we only have gst_value_fraction_subtract and not add, negate the
-       numerator */
+     * numerator */
+    GST_LOG_OBJECT (v4l2src, "stepwise step frame interval: %d/%d", num, denom);
     gst_value_set_fraction (&step, -num, denom);
 
-    num = ival.stepwise.max.numerator;
-    denom = ival.stepwise.max.denominator;
-    if (num > G_MAXINT || denom > G_MAXINT) {
-      num >>= 1;
-      denom >>= 1;
-    }
-    gst_value_set_fraction (&max, num, denom);
+    while (gst_value_compare (&min, &max) <= 0) {
+      GValue rate = { 0, };
 
-    while (gst_value_compare (&frac, &max) == GST_VALUE_LESS_THAN) {
-      GValue frac = { 0, };
-      g_value_init (&frac, GST_TYPE_FRACTION);
-      /* invert */
-      gst_value_set_fraction (&frac,
-          gst_value_get_fraction_denominator (&frac),
-          gst_value_get_fraction_numerator (&frac));
-      gst_value_list_append_value (&rate, &frac);
+      num = gst_value_get_fraction_numerator (&min);
+      denom = gst_value_get_fraction_denominator (&min);
+      GST_LOG_OBJECT (v4l2src, "adding stepwise framerate: %d/%d", denom, num);
 
-      if (!gst_value_fraction_subtract (&frac, &frac, &step)) {
-        GST_INFO_OBJECT (v4l2src, "could not step fraction!");
+      /* invert to get the framerate */
+      g_value_init (&rate, GST_TYPE_FRACTION);
+      gst_value_set_fraction (&rate, denom, num);
+      gst_value_list_append_value (&rates, &rate);
+      added = TRUE;
+
+      /* we're actually adding because step was negated above. This is because
+       * there is no _add function... */
+      if (!gst_value_fraction_subtract (&min, &min, &step)) {
+        GST_WARNING_OBJECT (v4l2src, "could not step fraction!");
         break;
       }
     }
+    if (!added) {
+      /* no range was added, leave the default range from the template */
+      GST_WARNING_OBJECT (v4l2src, "no range added, leaving default");
+      g_value_unset (&rates);
+    }
   } else if (ival.type == V4L2_FRMIVAL_TYPE_CONTINUOUS) {
     guint32 maxnum, maxdenom;
 
-    g_value_init (&rate, GST_TYPE_FRACTION_RANGE);
+    g_value_init (&rates, GST_TYPE_FRACTION_RANGE);
 
     num = ival.stepwise.min.numerator;
     denom = ival.stepwise.min.denominator;
@@ -519,35 +706,45 @@ gst_v4l2src_probe_caps_for_format_and_size (GstV4l2Src * v4l2src,
       denom >>= 1;
     }
 
-    maxnum = ival.stepwise.step.numerator;
-    maxdenom = ival.stepwise.step.denominator;
+    maxnum = ival.stepwise.max.numerator;
+    maxdenom = ival.stepwise.max.denominator;
     if (maxnum > G_MAXINT || maxdenom > G_MAXINT) {
       maxnum >>= 1;
       maxdenom >>= 1;
     }
 
-    gst_value_set_fraction_range_full (&rate, maxdenom, maxnum, denom, num);
+    GST_LOG_OBJECT (v4l2src, "continuous frame interval %d/%d to %d/%d",
+        maxdenom, maxnum, denom, num);
+
+    gst_value_set_fraction_range_full (&rates, maxdenom, maxnum, denom, num);
   } else {
     goto unknown_type;
   }
 
+return_data:
   s = gst_structure_copy (template);
   gst_structure_set (s, "width", G_TYPE_INT, (gint) width,
       "height", G_TYPE_INT, (gint) height, NULL);
-  gst_structure_set_value (s, "framerate", &rate);
-  g_value_unset (&rate);
+
+  if (G_IS_VALUE (&rates)) {
+    /* only change the framerate on the template when we have a valid probed new
+     * value */
+    gst_structure_set_value (s, "framerate", &rates);
+    g_value_unset (&rates);
+  }
   return s;
 
+  /* ERRORS */
 enum_frameintervals_failed:
   {
     GST_DEBUG_OBJECT (v4l2src,
-        "Failed to enumerate frame sizes for %" GST_FOURCC_FORMAT "@%ux%u",
+        "Unable to enumerate intervals for %" GST_FOURCC_FORMAT "@%ux%u",
         GST_FOURCC_ARGS (pixelformat), width, height);
-    return NULL;
+    goto return_data;
   }
 unknown_type:
   {
-    /* I don't see how this is actually an error */
+    /* I don't see how this is actually an error, we ignore the format then */
     GST_WARNING_OBJECT (v4l2src,
         "Unknown frame interval type at %" GST_FOURCC_FORMAT "@%ux%u: %u",
         GST_FOURCC_ARGS (pixelformat), width, height, ival.type);
@@ -560,15 +757,15 @@ GstCaps *
 gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
     const GstStructure * template)
 {
-#ifdef VIDIOC_ENUM_FRAMESIZES
-  GstCaps *ret;
+  GstCaps *ret = gst_caps_new_empty ();
   GstStructure *tmp;
+
+#ifdef VIDIOC_ENUM_FRAMESIZES
   gint fd = v4l2src->v4l2object->video_fd;
   struct v4l2_frmsizeenum size;
+  GList *results = NULL;
   guint32 w, h;
 
-  ret = gst_caps_new_empty ();
-
   memset (&size, 0, sizeof (struct v4l2_frmsizeenum));
   size.index = 0;
   size.pixel_format = pixelformat;
@@ -583,8 +780,11 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
 
       tmp = gst_v4l2src_probe_caps_for_format_and_size (v4l2src, pixelformat,
           w, h, template);
+
+      /* we get low res to high res, but want high res to low res in caps, so
+       * prepend structs to results list, we'll reverse the order later then */
       if (tmp)
-        gst_caps_append_structure (ret, tmp);
+        results = g_list_prepend (results, tmp);
 
       size.index++;
     } while (ioctl (fd, VIDIOC_ENUM_FRAMESIZES, &size) >= 0);
@@ -597,8 +797,11 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
 
       tmp = gst_v4l2src_probe_caps_for_format_and_size (v4l2src, pixelformat,
           w, h, template);
+
+      /* we get low res to high res, but want high res to low res in caps, so
+       * prepend structs to results list, we'll reverse the order later then */
       if (tmp)
-        gst_caps_append_structure (ret, tmp);
+        results = g_list_prepend (results, tmp);
     }
   } else if (size.type == V4L2_FRMSIZE_TYPE_CONTINUOUS) {
     guint32 maxw, maxh;
@@ -614,12 +817,28 @@ gst_v4l2src_probe_caps_for_format (GstV4l2Src * v4l2src, guint32 pixelformat,
       gst_structure_set (tmp, "width", GST_TYPE_INT_RANGE, (gint) w,
           (gint) maxw, "height", GST_TYPE_INT_RANGE, (gint) h, (gint) maxh,
           NULL);
+
+      /* no point using the results list here, since there's only one struct */
       gst_caps_append_structure (ret, tmp);
     }
   } else {
     goto unknown_type;
   }
 
+  /* we use an intermediary list to store the results of the probing because
+   * we probe from lowest resolution to highest resolution, but want the caps
+   * to contain the results in reverse order starting with the highest
+   * resolution, as order in caps matters for things like fixation.  However,
+   * there's no gst_caps_prepend_structure(), so we use the list as helper to
+   * reverse the order */
+  while (results != NULL) {
+    gst_caps_append_structure (ret, GST_STRUCTURE (results->data));
+    results = g_list_delete_link (results, results);
+  }
+
+  if (gst_caps_is_empty (ret))
+    goto enum_framesizes_no_results;
+
   return ret;
 
   /* ERRORS */
@@ -629,31 +848,77 @@ enum_framesizes_failed:
     GST_DEBUG_OBJECT (v4l2src,
         "Failed to enumerate frame sizes for pixelformat %" GST_FOURCC_FORMAT
         " (%s)", GST_FOURCC_ARGS (pixelformat), g_strerror (errno));
-    return NULL;
+    goto default_frame_sizes;
+  }
+enum_framesizes_no_results:
+  {
+    /* it's possible that VIDIOC_ENUM_FRAMESIZES is defined but the driver in
+     * question doesn't actually support it yet */
+    GST_DEBUG_OBJECT (v4l2src, "No results for pixelformat %" GST_FOURCC_FORMAT
+        " enumerating frame sizes, trying fallback",
+        GST_FOURCC_ARGS (pixelformat));
+    goto default_frame_sizes;
   }
 unknown_type:
   {
     GST_WARNING_OBJECT (v4l2src,
         "Unknown frame sizeenum type for pixelformat %" GST_FOURCC_FORMAT
         ": %u", GST_FOURCC_ARGS (pixelformat), size.type);
-    return NULL;
+    goto default_frame_sizes;
   }
-#else /* defined VIDIOC_ENUM_FRAMESIZES */
-  GstCaps *ret;
-  GstStructure *tmp;
-
-  /* This code is for Linux < 2.6.19 */
-
-  ret = gst_caps_new_empty ();
-  tmp = gst_structure_copy (template);
-  gst_structure_set (tmp, "width", GST_TYPE_INT_RANGE, (gint) 1,
-      (gint) GST_V4L2_MAX_SIZE, "height", GST_TYPE_INT_RANGE,
-      (gint) 1, (gint) GST_V4L2_MAX_SIZE,
-      "framerate", GST_TYPE_FRACTION_RANGE, (gint) 0,
-      (gint) 1, (gint) 100, (gint) 1, NULL);
-  gst_caps_append_structure (ret, tmp);
-  return ret;
+default_frame_sizes:
 #endif /* defined VIDIOC_ENUM_FRAMESIZES */
+  {
+    gint min_w, max_w, min_h, max_h, fix_num = 0, fix_denom = 0;
+
+    /* This code is for Linux < 2.6.19 */
+    min_w = min_h = 1;
+    max_w = max_h = GST_V4L2_MAX_SIZE;
+    if (!gst_v4l2src_get_nearest_size (v4l2src, pixelformat, &min_w, &min_h)) {
+      GST_WARNING_OBJECT (v4l2src,
+          "Could not probe minimum capture size for pixelformat %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (pixelformat));
+    }
+    if (!gst_v4l2src_get_nearest_size (v4l2src, pixelformat, &max_w, &max_h)) {
+      GST_WARNING_OBJECT (v4l2src,
+          "Could not probe maximum capture size for pixelformat %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (pixelformat));
+    }
+
+    /* Since we can't get framerate directly, try to use the current norm */
+    if (v4l2src->v4l2object->norm && v4l2src->v4l2object->norms) {
+      GList *norms;
+      GstTunerNorm *norm;
+
+      for (norms = v4l2src->v4l2object->norms; norms != NULL;
+          norms = norms->next) {
+        norm = (GstTunerNorm *) norms->data;
+        if (!strcmp (norm->label, v4l2src->v4l2object->norm))
+          break;
+      }
+      /* If it's possible, set framerate to that (discrete) value */
+      if (norm) {
+        fix_num = gst_value_get_fraction_numerator (&norm->framerate);
+        fix_denom = gst_value_get_fraction_denominator (&norm->framerate);
+      }
+    }
+
+    tmp = gst_structure_copy (template);
+    if (fix_num) {
+      gst_structure_set (tmp,
+          "width", GST_TYPE_INT_RANGE, min_w, max_w,
+          "height", GST_TYPE_INT_RANGE, min_h, max_h,
+          "framerate", GST_TYPE_FRACTION, fix_num, fix_denom, NULL);
+    } else {
+      /* if norm can't be used, copy the template framerate */
+      gst_structure_set (tmp,
+          "width", GST_TYPE_INT_RANGE, min_w, max_w,
+          "height", GST_TYPE_INT_RANGE, min_h, max_h, NULL);
+    }
+    gst_caps_append_structure (ret, tmp);
+
+    return ret;
+  }
 }
 
 /******************************************************
@@ -669,6 +934,7 @@ gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
   gint32 trials = NUM_TRIALS;
   GstBuffer *pool_buffer;
   gboolean need_copy;
+  gint index;
 
   memset (&buffer, 0x00, sizeof (buffer));
   buffer.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -710,9 +976,17 @@ gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
              else {
            */
           GST_DEBUG_OBJECT (v4l2src, "reenqueing buffer");
-          if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_QBUF, &buffer) < 0) {
-            goto qbuf_failed;
-          }
+          /* FIXME: this is not a good idea, as drivers usualy return the buffer
+           * with index-number set to 0, thus the re-enque will fail unless it
+           * was incidentialy 0.
+           * We could try to re-enque all buffers without handling the ioctl
+           * return.
+           */
+          /*
+             if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_QBUF, &buffer) < 0) {
+             goto qbuf_failed;
+             }
+           */
           /*} */
         }
         break;
@@ -740,11 +1014,24 @@ gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
 
   g_mutex_lock (v4l2src->pool->lock);
 
-  pool_buffer = GST_BUFFER (v4l2src->pool->buffers[buffer.index]);
+  index = buffer.index;
 
-  v4l2src->pool->buffers[buffer.index] = NULL;
+  /* get our GstBuffer with that index from the pool, if the buffer was
+   * outstanding we have a serious problem. */
+  pool_buffer = GST_BUFFER (v4l2src->pool->buffers[index]);
+
+  if (pool_buffer == NULL)
+    goto no_buffer;
+
+  GST_LOG_OBJECT (v4l2src, "grabbed buffer %p at index %d", pool_buffer, index);
+
+  /* we have the  buffer now, mark the spot in the pool empty */
+  v4l2src->pool->buffers[index] = NULL;
   v4l2src->pool->num_live_buffers++;
-  need_copy = v4l2src->pool->num_live_buffers == v4l2src->pool->buffer_count;
+  /* if we are handing out the last buffer in the pool, we need to make a
+   * copy and bring the buffer back in the pool. */
+  need_copy = v4l2src->always_copy
+      || (v4l2src->pool->num_live_buffers == v4l2src->pool->buffer_count);
 
   g_mutex_unlock (v4l2src->pool->lock);
 
@@ -782,14 +1069,16 @@ gst_v4l2src_grab_frame (GstV4l2Src * v4l2src, GstBuffer ** buf)
 
   if (G_UNLIKELY (need_copy)) {
     *buf = gst_buffer_copy (pool_buffer);
+    GST_BUFFER_FLAG_UNSET (*buf, GST_BUFFER_FLAG_READONLY);
     /* this will requeue */
     gst_buffer_unref (pool_buffer);
   } else {
     *buf = pool_buffer;
   }
 
-  GST_LOG_OBJECT (v4l2src, "grabbed frame %d (ix=%d), pool-ct=%d",
-      buffer.sequence, buffer.index, v4l2src->pool->num_live_buffers);
+  GST_LOG_OBJECT (v4l2src, "grabbed frame %d (ix=%d), flags %08x, pool-ct=%d",
+      buffer.sequence, buffer.index, buffer.flags,
+      v4l2src->pool->num_live_buffers);
 
   return GST_FLOW_OK;
 
@@ -820,6 +1109,16 @@ too_many_trials:
             NUM_TRIALS, v4l2src->v4l2object->videodev, g_strerror (errno)));
     return GST_FLOW_ERROR;
   }
+no_buffer:
+  {
+    GST_ELEMENT_ERROR (v4l2src, RESOURCE, FAILED,
+        (_("Failed trying to get video frames from device '%s'."),
+            v4l2src->v4l2object->videodev),
+        (_("No free buffers found in the pool at index %d."), index));
+    g_mutex_unlock (v4l2src->pool->lock);
+    return GST_FLOW_ERROR;
+  }
+/*
 qbuf_failed:
   {
     GST_ELEMENT_ERROR (v4l2src, RESOURCE, WRITE,
@@ -829,8 +1128,22 @@ qbuf_failed:
             v4l2src->v4l2object->videodev, g_strerror (errno)));
     return GST_FLOW_ERROR;
   }
+*/
 }
 
+static gboolean
+fractions_are_equal (gint num1, gint den1, gint num2, gint den2)
+{
+  GValue fraction1 = { 0, }, fraction2 = {
+  0,};
+
+  g_value_init (&fraction1, GST_TYPE_FRACTION);
+  g_value_init (&fraction2, GST_TYPE_FRACTION);
+  gst_value_set_fraction (&fraction1, num1, den1);
+  gst_value_set_fraction (&fraction2, num2, den2);
+  /* we know we don't have to unset the values in this case */
+  return (gst_value_compare (&fraction1, &fraction2) == GST_VALUE_EQUAL);
+}
 
 /******************************************************
  * gst_v4l2src_set_capture():
@@ -845,8 +1158,8 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   struct v4l2_format format;
   struct v4l2_streamparm stream;
 
-  DEBUG ("Setting capture format to %dx%d, format %" GST_FOURCC_FORMAT,
-      width, height, GST_FOURCC_ARGS (pixelformat));
+  GST_DEBUG_OBJECT (v4l2src, "Setting capture format to %dx%d, format "
+      "%" GST_FOURCC_FORMAT, width, height, GST_FOURCC_ARGS (pixelformat));
 
   GST_V4L2_CHECK_OPEN (v4l2src->v4l2object);
   GST_V4L2_CHECK_NOT_ACTIVE (v4l2src->v4l2object);
@@ -861,11 +1174,23 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   format.fmt.pix.width = width;
   format.fmt.pix.height = height;
   format.fmt.pix.pixelformat = pixelformat;
-  /* request whole frames; change when gstreamer supports interlaced video */
+  /* request whole frames; change when gstreamer supports interlaced video
+   * (INTERLACED mode returns frames where the fields have already been
+   *  combined, there are other modes for requesting fields individually) */
   format.fmt.pix.field = V4L2_FIELD_INTERLACED;
 
-  if (ioctl (fd, VIDIOC_S_FMT, &format) < 0)
-    goto set_fmt_failed;
+  if (ioctl (fd, VIDIOC_S_FMT, &format) < 0) {
+    if (errno != EINVAL)
+      goto set_fmt_failed;
+
+    /* try again with progressive video */
+    format.fmt.pix.width = width;
+    format.fmt.pix.height = height;
+    format.fmt.pix.pixelformat = pixelformat;
+    format.fmt.pix.field = V4L2_FIELD_NONE;
+    if (ioctl (fd, VIDIOC_S_FMT, &format) < 0)
+      goto set_fmt_failed;
+  }
 
   if (format.fmt.pix.width != width || format.fmt.pix.height != height)
     goto invalid_dimensions;
@@ -873,23 +1198,51 @@ gst_v4l2src_set_capture (GstV4l2Src * v4l2src, guint32 pixelformat,
   if (format.fmt.pix.pixelformat != pixelformat)
     goto invalid_pixelformat;
 
+  /* Is there a reason we require the caller to always specify a framerate? */
+  GST_LOG_OBJECT (v4l2src, "Desired framerate: %u/%u", fps_n, fps_d);
+
   memset (&stream, 0x00, sizeof (struct v4l2_streamparm));
   stream.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-  if (ioctl (fd, VIDIOC_G_PARM, &stream) < 0)
-    goto get_parm_failed;
+  if (ioctl (fd, VIDIOC_G_PARM, &stream) < 0) {
+    GST_ELEMENT_WARNING (v4l2src, RESOURCE, SETTINGS,
+        (_("Could not get parameters on device '%s'"),
+            v4l2src->v4l2object->videodev), GST_ERROR_SYSTEM);
+    goto done;
+  }
 
-  /* Note: V4L2 gives us the frame interval, we need the frame rate */
+  /* Note: V4L2 provides the frame interval, we have the frame rate */
+  if (fractions_are_equal (stream.parm.capture.timeperframe.numerator,
+          stream.parm.capture.timeperframe.denominator, fps_d, fps_n)) {
+    GST_LOG_OBJECT (v4l2src, "Desired framerate already set, nothing to do");
+    goto done;
+  }
+
+  /* We want to change the frame rate, so check whether we can. Some cheap USB
+   * cameras don't have the capability */
+  if ((stream.parm.capture.capability & V4L2_CAP_TIMEPERFRAME) == 0) {
+    GST_DEBUG_OBJECT (v4l2src, "Not setting framerate (not supported)");
+    goto done;
+  }
+
+  GST_LOG_OBJECT (v4l2src, "Setting framerate to %u/%u", fps_n, fps_d);
+
+  /* Note: V4L2 wants the frame interval, we have the frame rate */
   stream.parm.capture.timeperframe.numerator = fps_d;
   stream.parm.capture.timeperframe.denominator = fps_n;
 
-  if (ioctl (fd, VIDIOC_S_PARM, &stream) < 0)
-    goto set_parm_failed;
+  /* some cheap USB cam's won't accept any change */
+  if (ioctl (fd, VIDIOC_S_PARM, &stream) < 0) {
+    GST_ELEMENT_WARNING (v4l2src, RESOURCE, SETTINGS,
+        (_("Video input device did not accept new frame rate setting.")),
+        GST_ERROR_SYSTEM);
+    goto done;
+  }
 
+  v4l2src->fps_n = fps_n;
+  v4l2src->fps_d = fps_d;
+  GST_INFO_OBJECT (v4l2src, "Set framerate to %u/%u", fps_n, fps_d);
 
-  /* FIXME: better test for fraction equality */
-  if (stream.parm.capture.timeperframe.numerator != fps_d
-      || stream.parm.capture.timeperframe.denominator != fps_n)
-    goto invalid_framerate;
+done:
 
   return TRUE;
 
@@ -931,30 +1284,6 @@ invalid_pixelformat:
             GST_FOURCC_ARGS (format.fmt.pix.pixelformat)));
     return FALSE;
   }
-get_parm_failed:
-  {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SETTINGS,
-        (_("Could not get parameters on device '%s'"),
-            v4l2src->v4l2object->videodev), GST_ERROR_SYSTEM);
-    return FALSE;
-  }
-set_parm_failed:
-  {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SETTINGS,
-        (_("Could not set parameters on device '%s'"),
-            v4l2src->v4l2object->videodev), GST_ERROR_SYSTEM);
-    return FALSE;
-  }
-invalid_framerate:
-  {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SETTINGS,
-        (_("Device '%s' cannot capture at %d/%d frames per second"),
-            v4l2src->v4l2object->videodev, fps_n, fps_d),
-        ("Tried to capture at %d/%d fps, but device returned %d/%d fps",
-            fps_n, fps_d, stream.parm.capture.timeperframe.denominator,
-            stream.parm.capture.timeperframe.numerator));
-    return FALSE;
-  }
 }
 
 /******************************************************
@@ -976,24 +1305,30 @@ gst_v4l2src_capture_init (GstV4l2Src * v4l2src, GstCaps * caps)
   GST_V4L2_CHECK_NOT_ACTIVE (v4l2src->v4l2object);
 
   if (v4l2src->v4l2object->vcap.capabilities & V4L2_CAP_STREAMING) {
-    breq.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    breq.count = v4l2src->num_buffers;
 
+    GST_DEBUG_OBJECT (v4l2src, "STREAMING, requesting %d MMAP CAPTURE buffers",
+        v4l2src->num_buffers);
+
+    breq.count = v4l2src->num_buffers;
+    breq.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     breq.memory = V4L2_MEMORY_MMAP;
+
     if (ioctl (fd, VIDIOC_REQBUFS, &breq) < 0)
       goto reqbufs_failed;
 
+    GST_LOG_OBJECT (v4l2src, " count:  %u", breq.count);
+    GST_LOG_OBJECT (v4l2src, " type:   %d", breq.type);
+    GST_LOG_OBJECT (v4l2src, " memory: %d", breq.memory);
+
     if (breq.count < GST_V4L2_MIN_BUFFERS)
       goto no_buffers;
 
     if (v4l2src->num_buffers != breq.count) {
+      GST_WARNING_OBJECT (v4l2src, "using %u buffers instead", breq.count);
       v4l2src->num_buffers = breq.count;
       g_object_notify (G_OBJECT (v4l2src), "queue-size");
     }
 
-    GST_INFO_OBJECT (v4l2src, "Got %d buffers of size %d kB",
-        breq.count, v4l2src->frame_byte_size / 1024);
-
     /* Map the buffers */
     GST_LOG_OBJECT (v4l2src, "initiating buffer pool");
 
@@ -1169,54 +1504,77 @@ gst_v4l2src_capture_deinit (GstV4l2Src * v4l2src)
 
 /*
  */
-gboolean
-gst_v4l2src_get_size_limits (GstV4l2Src * v4l2src,
-    struct v4l2_fmtdesc * format,
-    gint * min_w, gint * max_w, gint * min_h, gint * max_h)
+static gboolean
+gst_v4l2src_get_nearest_size (GstV4l2Src * v4l2src, guint32 pixelformat,
+    gint * width, gint * height)
 {
-
   struct v4l2_format fmt;
+  int fd;
+  int r;
+
+  g_return_val_if_fail (width != NULL, FALSE);
+  g_return_val_if_fail (height != NULL, FALSE);
 
   GST_LOG_OBJECT (v4l2src,
-      "getting size limits with format %" GST_FOURCC_FORMAT,
-      GST_FOURCC_ARGS (format->pixelformat));
+      "getting nearest size to %dx%d with format %" GST_FOURCC_FORMAT,
+      *width, *height, GST_FOURCC_ARGS (pixelformat));
+
+  fd = v4l2src->v4l2object->video_fd;
 
   /* get size delimiters */
   memset (&fmt, 0, sizeof (fmt));
   fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-  fmt.fmt.pix.width = 0;
-  fmt.fmt.pix.height = 0;
-  fmt.fmt.pix.pixelformat = format->pixelformat;
+  fmt.fmt.pix.width = *width;
+  fmt.fmt.pix.height = *height;
+  fmt.fmt.pix.pixelformat = pixelformat;
   fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
-  if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_TRY_FMT, &fmt) < 0) {
-    GST_DEBUG_OBJECT (v4l2src, "failed to get min size: %s",
-        g_strerror (errno));
-    return FALSE;
+
+  r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
+  if (r < 0 && errno == EINVAL) {
+    /* try again with progressive video */
+    fmt.fmt.pix.width = *width;
+    fmt.fmt.pix.height = *height;
+    fmt.fmt.pix.pixelformat = pixelformat;
+    fmt.fmt.pix.field = V4L2_FIELD_NONE;
+    r = ioctl (fd, VIDIOC_TRY_FMT, &fmt);
   }
 
-  if (min_w)
-    *min_w = fmt.fmt.pix.width;
-  if (min_h)
-    *min_h = fmt.fmt.pix.height;
+  if (r < 0) {
+    /* The driver might not implement TRY_FMT, in which case we will try
+       S_FMT to probe */
+    if (errno != ENOTTY)
+      return FALSE;
 
-  GST_LOG_OBJECT (v4l2src,
-      "got min size %dx%d", fmt.fmt.pix.width, fmt.fmt.pix.height);
+    /* Only try S_FMT if we're not actively capturing yet, which we shouldn't
+       be, because we're still probing */
+    if (GST_V4L2_IS_ACTIVE (v4l2src->v4l2object))
+      return FALSE;
 
-  fmt.fmt.pix.width = GST_V4L2_MAX_SIZE;
-  fmt.fmt.pix.height = GST_V4L2_MAX_SIZE;
-  if (ioctl (v4l2src->v4l2object->video_fd, VIDIOC_TRY_FMT, &fmt) < 0) {
-    GST_DEBUG_OBJECT (v4l2src, "failed to get max size: %s",
-        g_strerror (errno));
-    return FALSE;
+    GST_LOG_OBJECT (v4l2src,
+        "Failed to probe size limit with VIDIOC_TRY_FMT, trying VIDIOC_S_FMT");
+
+    fmt.fmt.pix.width = *width;
+    fmt.fmt.pix.height = *height;
+
+    r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+    if (r < 0 && errno == EINVAL) {
+      /* try again with progressive video */
+      fmt.fmt.pix.width = *width;
+      fmt.fmt.pix.height = *height;
+      fmt.fmt.pix.pixelformat = pixelformat;
+      fmt.fmt.pix.field = V4L2_FIELD_NONE;
+      r = ioctl (fd, VIDIOC_S_FMT, &fmt);
+    }
+
+    if (r < 0)
+      return FALSE;
   }
 
-  if (max_w)
-    *max_w = fmt.fmt.pix.width;
-  if (max_h)
-    *max_h = fmt.fmt.pix.height;
-
   GST_LOG_OBJECT (v4l2src,
-      "got max size %dx%d", fmt.fmt.pix.width, fmt.fmt.pix.height);
+      "got nearest size %dx%d", fmt.fmt.pix.width, fmt.fmt.pix.height);
+
+  *width = fmt.fmt.pix.width;
+  *height = fmt.fmt.pix.height;
 
   return TRUE;
 }
