@@ -161,6 +161,11 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
     int error_base;
     long evmask = NoEventMask;
 
+    s->have_xdamage = FALSE;
+    s->damage = None;
+    s->damage_copy_gc = None;
+    s->damage_region = None;
+
     if (XDamageQueryExtension (s->xcontext->disp, &s->damage_event_base,
             &error_base)) {
       s->damage =
@@ -219,6 +224,22 @@ gst_ximage_src_stop (GstBaseSrc * basesrc)
 
   if (src->xcontext) {
     g_mutex_lock (src->x_lock);
+
+#ifdef HAVE_XDAMAGE
+    if (src->damage_copy_gc != None) {
+      XFreeGC (src->xcontext->disp, src->damage_copy_gc);
+      src->damage_copy_gc = None;
+    }
+    if (src->damage_region != None) {
+      XFixesDestroyRegion (src->xcontext->disp, src->damage_region);
+      src->damage_region = None;
+    }
+    if (src->damage != None) {
+      XDamageDestroy (src->xcontext->disp, src->damage);
+      src->damage = None;
+    }
+#endif
+
     ximageutil_xcontext_clear (src->xcontext);
     src->xcontext = NULL;
     g_mutex_unlock (src->x_lock);
@@ -280,7 +301,7 @@ composite_pixel (GstXContext * xcontext, guchar * dest, guchar * src)
       break;
     default:
       /* Should not reach here */
-      g_assert_not_reached ();
+      g_return_if_reached ();
   }
 
   /* possible optimisation:
