@@ -40,7 +40,7 @@ video_crop_get_test_caps (GstElement * videocrop)
   GList *list = NULL;
   guint i;
 
-  srcpad = gst_element_get_pad (videocrop, "src");
+  srcpad = gst_element_get_static_pad (videocrop, "src");
   fail_unless (srcpad != NULL);
   allowed_caps = gst_pad_get_pad_template_caps (srcpad);
   fail_unless (allowed_caps != NULL);
@@ -74,7 +74,6 @@ GST_START_TEST (test_unit_sizes)
   GstBaseTransformClass *csp_klass, *vcrop_klass;
   GstElement *videocrop, *csp;
   GList *caps_list, *l;
-  gint i;
 
   videocrop = gst_element_factory_make ("videocrop", "videocrop");
   fail_unless (videocrop != NULL, "Failed to create videocrop element");
@@ -111,7 +110,6 @@ GST_START_TEST (test_unit_sizes)
 
     for (i = 0; i < G_N_ELEMENTS (sizes_to_try); ++i) {
       gchar *caps_str;
-      guint32 format = 0;
       guint csp_size = 0;
       guint vc_size = 0;
 
@@ -398,7 +396,7 @@ GST_START_TEST (test_cropping)
 
   for (node = caps_list; node != NULL; node = node->next) {
     GstStructure *s;
-    GstCaps *caps;
+    GstCaps *caps, *in_caps;
 
     caps = gst_caps_copy (GST_CAPS (node->data));
     s = gst_caps_get_structure (caps, 0);
@@ -407,34 +405,36 @@ GST_START_TEST (test_cropping)
     GST_INFO ("testing format: %" GST_PTR_FORMAT, caps);
 
     for (i = 0; i < G_N_ELEMENTS (sizes_to_try); ++i) {
-      gst_structure_set (s, "width", G_TYPE_INT, sizes_to_try[i].width,
-          "height", G_TYPE_INT, sizes_to_try[i].height, NULL);
-
       GST_INFO (" - %d x %d", sizes_to_try[i].width, sizes_to_try[i].height);
 
-      videocrop_test_cropping (&ctx, caps, 0, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 1, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 1, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 0, 1, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 0, 0, 1, NULL);
-      videocrop_test_cropping (&ctx, caps, 63, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 63, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 0, 63, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 0, 0, 63, NULL);
-      videocrop_test_cropping (&ctx, caps, 63, 0, 0, 1, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 63, 1, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 1, 63, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 1, 0, 0, 63, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 32, 0, 0, 128, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 32, 128, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 0, 128, 32, 0, NULL);
-      videocrop_test_cropping (&ctx, caps, 128, 0, 0, 32, NULL);
-      videocrop_test_cropping (&ctx, caps, 1, 1, 1, 1, NULL);
-      videocrop_test_cropping (&ctx, caps, 63, 63, 63, 63, NULL);
-      videocrop_test_cropping (&ctx, caps, 64, 64, 64, 64, NULL);
-    }
+      gst_structure_set (s, "width", G_TYPE_INT, sizes_to_try[i].width,
+          "height", G_TYPE_INT, sizes_to_try[i].height, NULL);
+      in_caps = gst_caps_copy (caps);
 
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 1, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 1, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 1, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 63, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 63, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 63, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 1, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 63, 1, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 1, 63, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 63, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 32, 0, 0, 128, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 32, 128, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 0, 128, 32, 0, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 128, 0, 0, 32, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 1, 1, 1, 1, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 63, 63, 63, 63, NULL);
+      videocrop_test_cropping (&ctx, in_caps, 64, 64, 64, 64, NULL);
+
+      gst_caps_unref (in_caps);
+    }
     gst_caps_unref (caps);
   }
   g_list_foreach (caps_list, (GFunc) gst_caps_unref, NULL);
@@ -464,7 +464,7 @@ GST_START_TEST (test_passthrough)
 
   g_object_set (ctx.src, "num-buffers", 1, NULL);
 
-  srcpad = gst_element_get_pad (ctx.src, "src");
+  srcpad = gst_element_get_static_pad (ctx.src, "src");
   fail_unless (srcpad != NULL);
   gst_pad_add_buffer_probe (srcpad, G_CALLBACK (buffer_probe_cb), &gen_buf);
   gst_object_unref (srcpad);
